import logging
import pathlib

import yaml
from django.core.management.base import BaseCommand
from kitchenai.core.models import KitchenAIDependencies
from kitchenai.core.models import KitchenAIManagement
from kitchenai.core.models import KitchenAIModules
from kitchenai.core.models import KitchenAIPlugins

logger = logging.getLogger("kitchenai.core.commands")


class Command(BaseCommand):
    help = 'Inits kitchenai'

    def handle(self, *args, **options):
        """Checks to see if there is a kitchen.yml in the root directory, loads and initializes the db"""

        config = pathlib.Path.cwd() / "kitchenai.yml"
        config_data = self.read_yaml_config(config)

        if config_data:
            # Save the configuration to the database
            self.save_config_to_db(config_data)
            self.stdout.write(self.style.SUCCESS('KitchenAI initialized and configuration saved successfully.'))
        else:
            self.stdout.write(self.style.ERROR('No config file for kitchenai.'))

    def read_yaml_config(self, yaml_file_path):

        # Check if the file exists
        if not yaml_file_path.is_file():
            self.stdout.write(self.style.ERROR("YAML config file kitchenai.yml not found."))
            return None

        # Read the YAML file
        with open(yaml_file_path) as file:
            try:
                return yaml.safe_load(file)
            except yaml.YAMLError as e:
                self.stdout.write(self.style.ERROR(f"Error reading YAML file: {e}"))
                return None

    def save_config_to_db(self, config):
        # Clear existing config
        KitchenAIManagement.objects.all().delete()
        try:
            mgmt = KitchenAIManagement.objects.create(
                version = config["version"],
                project_name = config["project_name"]
            )
        except Exception as e:
            logger.error(e)
            return

        # Save the main app module
        try:
            app = config["app"]
            KitchenAIModules.objects.create(
                name=app,
                kitchen = mgmt,
                is_root = True
            )
            self.stdout.write(self.style.SUCCESS(f"main app loaded: {app}"))

        except KeyError as e:
            logger.error(e)
            return
        except Exception as e:
            logger.error(e)
            return

        #installed apps i.e cookbooks
        try:
            installed_apps = config["installed_apps"]
            for app in installed_apps:
                KitchenAIModules.objects.create(
                    name=app,
                    kitchen = mgmt,
                )
        except KeyError as e:
            logger.error(e)
            return
        except Exception as e:
            logger.error(e)
            return

        try:
            plugins = config.get("plugins", None)
            if plugins:
                for plugin in plugins:
                    KitchenAIPlugins.objects.create(
                        name=plugin,
                        kitchen = mgmt,
                    )
        except KeyError as e:
            logger.error(e)
            return
        except Exception as e:
            logger.error(e)
            return

        try:
            dependencies = config.get("dependencies", None)
            if dependencies:
                for dep in dependencies:
                    KitchenAIDependencies.objects.create(
                        name=dep,
                        kitchen = mgmt,
                    )
        except KeyError as e:
            logger.error(e)
            return
        except Exception as e:
            logger.error(e)
            return



import logging
import os
import sys
from importlib import import_module
from typing import TYPE_CHECKING

from django.apps import apps
from django.conf import settings
from kitchenai.contrib.kitchenai_sdk.kitchenai import KitchenAIApp
from kitchenai.core.models import KitchenAIManagement
from kitchenai.core.models import KitchenAIRootModule

if TYPE_CHECKING:
    from ninja import NinjaAPI

logger = logging.getLogger("kitchenai.core.utils")


def update_installed_apps(self, apps):
    if apps:
        settings.INSTALLED_APPS += tuple(apps)
        self.stdout.write(self.style.SUCCESS(f'Updated INSTALLED_APPS: {settings.INSTALLED_APPS}'))

def import_modules(module_paths):
    for name, path in module_paths.items():
        try:
            module_path, instance_name = path.split(':')
            module = import_module(module_path)
            instance = getattr(module, instance_name)
            globals()[name] = instance
            logger.info(f'Imported {instance_name} from {module_path}')
        except (ImportError, AttributeError) as e:
            logger.error(f"Error loading module '{path}': {e}")

def import_cookbook(module_path):
    try:
        module_path, instance_name = module_path.split(':')
        module = import_module(module_path)
        instance = getattr(module, instance_name)
        print(f'Imported {instance_name} from {module_path}')
        return instance
    except (ImportError, AttributeError) as e:
        print(f"Error loading module '{e}")



def setup(api: "NinjaAPI", module: str = "", project_root: str = os.getcwd()) -> "KitchenAIApp":
    # # Load configuration from the database
    config = {}
    # Determine the user's project root directory (assumes the command is run from the user's project root)
    # Add the user's project root directory to the Python path
    if project_root not in sys.path:
        sys.path.insert(0, project_root)

    if module:
        logger.debug(f"importing module: {module}")
        config["app"] = module

        #add module to db
        root_module = get_or_create_root_module(config["app"])
        if root_module:
            root_module.name = config["app"]
            root_module.save()
    else:
        logger.info("No module found in command or config. Running without any dynamic modules")
        return

    add_module_to_core(config["app"])


def add_module_router(api: "NinjaAPI"):
    """
    Add a router to the api
    """
    core_app = apps.get_app_config("core")
    if core_app.kitchenai_app:
        core_app.kitchenai_app.register_api()
        api.add_router(f"/{core_app.kitchenai_app._namespace}", core_app.kitchenai_app._router)
    else:
        logger.error("No kitchenai app in core app config")
        return

def add_module_to_core(module_path: str):
    """
    Add a module to the core app
    """
    #importing main app
    try:
        module_path, instance_name = module_path.split(':')

        print(f"module_path in add_module_to_core: {module_path}, instance name: {instance_name}")
        module = import_module(module_path)
        instance = getattr(module, instance_name)

        logger.info(f'Imported {instance_name} from {module_path}')
        if isinstance(instance, KitchenAIApp):
            #add the instance to the core app
            core_app = apps.get_app_config("core")
            core_app.kitchenai_app = instance
            logger.info(f'{instance_name} is a valid KitchenAIApp instance.')
        else:
            logger.error(f'{instance_name} is not a valid KitchenAIApp instance.')
        return instance

    except (ImportError, AttributeError) as e:
        logger.warning(f"No valid KitchenAIApp instance found: {e}")

    except ValueError as e:
        logger.error(f"Invalid module path format. Expected 'module:instance' {e}")

    except Exception as e:
        logger.error(f"error adding module to core: {e}")

def get_or_create_root_module(module_path: str):
    try:
        return KitchenAIRootModule.objects.get(name=module_path)
    except KitchenAIRootModule.DoesNotExist:
        kitchen_mgmt = KitchenAIManagement.objects.get(name="kitchenai_management")
        #create a new root module
        root_module = KitchenAIRootModule(name=module_path, kitchen=kitchen_mgmt)
        root_module.save()
        logger.warning(f"No root module found for {module_path}. Created new root module.")

def get_first_root_module() -> str:
    try:
        return KitchenAIRootModule.objects.first().name
    except KitchenAIRootModule.DoesNotExist:
        raise Exception("No root module found. Please create a root module first.")


def get_core_kitchenai_app():
    project_root = os.getcwd()

    # Add the user's project root directory to the Python path
    if project_root not in sys.path:
        sys.path.insert(0, project_root)
    root_module = get_first_root_module()

    print(f"root_module: {root_module}")
    add_module_to_core(root_module)
    core_app = apps.get_app_config("core")
    if core_app.kitchenai_app:
        return core_app.kitchenai_app
    else:
        logger.error("No kitchenai app in core app config")
        raise Exception("No kitchenai app in core app config")



# Configuration file for the Sphinx documentation builder.
#
# For the full list of built-in configuration values, see the documentation:
# https://www.sphinx-doc.org/en/master/usage/configuration.html
import os
import sys

import django

# -- Path setup --------------------------------------------------------------
# If extensions (or modules to document with autodoc) are in another directory,
# add these directories to sys.path here. If the directory is relative to the
# documentation root, use os.path.abspath to make it absolute, like shown here.
#

sys.path.insert(0, os.path.abspath(".."))


# -- Django setup -----------------------------------------------------------
# This is required to import Django code in Sphinx using autodoc.

os.environ["DJANGO_SETTINGS_MODULE"] = "kitchenai.settings"
django.setup()

# -- Project information -----------------------------------------------------
# https://www.sphinx-doc.org/en/master/usage/configuration.html#project-information

project = 'kitchenai'
copyright = '2024, epuerta'
author = 'epuerta'
release = '2024'

# -- General configuration ---------------------------------------------------
# https://www.sphinx-doc.org/en/master/usage/configuration.html#general-configuration

extensions = [
    "myst_parser",
    "sphinx_copybutton",
    "sphinx.ext.autodoc",
    "sphinx_design",
]

templates_path = ['_templates']
exclude_patterns = ['_build', 'Thumbs.db', '.DS_Store']


# -- Options for HTML output -------------------------------------------------
# https://www.sphinx-doc.org/en/master/usage/configuration.html#options-for-html-output

html_theme = 'furo'
html_title = project
html_static_path = ['_static']



# Generated by Django 5.1.2 on 2024-11-27 01:42

import django.db.models.deletion
import kitchenai.core.models
from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = []

    operations = [
        migrations.CreateModel(
            name="EmbedObject",
            fields=[
                (
                    "id",
                    models.BigAutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("created_at", models.DateTimeField(auto_now_add=True)),
                ("updated_at", models.DateTimeField(auto_now=True)),
                ("text", models.CharField(max_length=255)),
                ("ingest_label", models.CharField(max_length=255)),
                ("status", models.CharField(default="pending", max_length=255)),
                ("metadata", models.JSONField(default=dict)),
            ],
            options={
                "abstract": False,
            },
        ),
        migrations.CreateModel(
            name="FileObject",
            fields=[
                (
                    "id",
                    models.BigAutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("created_at", models.DateTimeField(auto_now_add=True)),
                ("updated_at", models.DateTimeField(auto_now=True)),
                (
                    "file",
                    models.FileField(
                        upload_to=kitchenai.core.models.file_object_directory_path
                    ),
                ),
                ("name", models.CharField(max_length=255)),
                ("ingest_label", models.CharField(max_length=255)),
                ("status", models.CharField(default="pending", max_length=255)),
                ("metadata", models.JSONField(default=dict)),
            ],
            options={
                "abstract": False,
            },
        ),
        migrations.CreateModel(
            name="KitchenAIManagement",
            fields=[
                ("created_at", models.DateTimeField(auto_now_add=True)),
                ("updated_at", models.DateTimeField(auto_now=True)),
                (
                    "name",
                    models.CharField(
                        default="kitchenai_management",
                        max_length=255,
                        primary_key=True,
                        serialize=False,
                    ),
                ),
                ("project_name", models.CharField(max_length=255)),
                ("version", models.CharField(max_length=255)),
                ("description", models.TextField(default="")),
                ("jupyter_token", models.CharField(default="", max_length=255)),
                ("jupyter_host", models.CharField(default="", max_length=255)),
                ("jupyter_port", models.CharField(default="8888", max_length=255)),
                ("jupyter_protocol", models.CharField(default="http", max_length=255)),
            ],
            options={
                "abstract": False,
            },
        ),
        migrations.CreateModel(
            name="KitchenAIDependencies",
            fields=[
                (
                    "id",
                    models.BigAutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("created_at", models.DateTimeField(auto_now_add=True)),
                ("updated_at", models.DateTimeField(auto_now=True)),
                ("name", models.CharField(max_length=255, unique=True)),
                (
                    "kitchen",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        to="core.kitchenaimanagement",
                    ),
                ),
            ],
            options={
                "abstract": False,
            },
        ),
        migrations.CreateModel(
            name="KitchenAIModule",
            fields=[
                (
                    "id",
                    models.BigAutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("created_at", models.DateTimeField(auto_now_add=True)),
                ("updated_at", models.DateTimeField(auto_now=True)),
                ("name", models.CharField(max_length=255)),
                ("jupyter_path", models.CharField(default="", max_length=255)),
                (
                    "file",
                    models.FileField(
                        upload_to=kitchenai.core.models.module_directory_path
                    ),
                ),
                (
                    "kitchen",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        to="core.kitchenaimanagement",
                    ),
                ),
            ],
            options={
                "abstract": False,
            },
        ),
        migrations.CreateModel(
            name="KitchenAIPlugins",
            fields=[
                (
                    "id",
                    models.BigAutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("created_at", models.DateTimeField(auto_now_add=True)),
                ("updated_at", models.DateTimeField(auto_now=True)),
                ("name", models.CharField(max_length=255, unique=True)),
                (
                    "kitchen",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        to="core.kitchenaimanagement",
                    ),
                ),
            ],
            options={
                "abstract": False,
            },
        ),
        migrations.CreateModel(
            name="KitchenAIRootModule",
            fields=[
                (
                    "id",
                    models.BigAutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("created_at", models.DateTimeField(auto_now_add=True)),
                ("updated_at", models.DateTimeField(auto_now=True)),
                ("name", models.CharField(max_length=255, unique=True)),
                (
                    "kitchen",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        to="core.kitchenaimanagement",
                    ),
                ),
            ],
            options={
                "abstract": False,
            },
        ),
    ]



from django.apps import AppConfig
import posthog

class CoreConfig(AppConfig):
    default_auto_field = "django.db.models.BigAutoField"
    name = "kitchenai.core"
    kitchenai_app = None

    def ready(self):
        from . import signals

        posthog.api_key = 'phc_9X7VLQwkV5h90fb6DK85rk5uesGarhFfdf7vWc7AEQG'
        posthog.host = 'https://us.i.posthog.com'
        


import logging
import os

import pytest
from django.conf import settings
from django.core.files.uploadedfile import SimpleUploadedFile
from django.test.utils import override_settings

pytest_plugins = []  # type: ignore


def pytest_configure(config):
    logging.disable(logging.CRITICAL)


TEST_SETTINGS = {
    "DEBUG": False,
    "CACHES": {
        "default": {
            "BACKEND": "django.core.cache.backends.dummy.DummyCache",
        }
    },
    "EMAIL_BACKEND": "django.core.mail.backends.locmem.EmailBackend",
    "LOGGING_CONFIG": None,
    "PASSWORD_HASHERS": [
        "django.contrib.auth.hashers.MD5PasswordHasher",
    ],
    "Q_CLUSTER": {
        "sync": True,
    },
    "STORAGES": {
        "default": {
            "BACKEND": "django.core.files.storage.InMemoryStorage",
        },
        "staticfiles": {
            "BACKEND": "django.contrib.staticfiles.storage.StaticFilesStorage",
        },
    },
    "WHITENOISE_AUTOREFRESH": True,
}


@pytest.fixture(autouse=True, scope="session")
def use_test_settings():
    with override_settings(**TEST_SETTINGS):
        yield


@pytest.fixture(autouse=True)
def setup_test_environment():
    # Setup test environment variables
    os.environ["OPENAI_API_KEY"] = "test-key"

    # Configure Django settings
    settings.configure(
        DEBUG=True,
        DATABASES={
            'default': {
                'ENGINE': 'django.db.backends.sqlite3',
                'NAME': ':memory:',
            }
        },
        INSTALLED_APPS=[
            'django.contrib.auth',
            'django.contrib.contenttypes',
            'kitchenai.core',
        ],
        MIDDLEWARE=[
            'django.middleware.security.SecurityMiddleware',
            'django.middleware.common.CommonMiddleware',
        ],
        ROOT_URLCONF='tests.urls',
        SECRET_KEY='test-key',
    )


@pytest.fixture
def test_file_content():
    return "This is a test document for vector storage"


@pytest.fixture
def uploaded_file(test_file_content):
    return SimpleUploadedFile(
        "test.txt",
        test_file_content.encode(),
        content_type="text/plain"
    )



import djp
from allauth.account.decorators import secure_admin_login
from django.conf import settings
from django.conf.urls.static import static
from django.contrib import admin
from django.urls import include
from django.urls import path
from django.views import defaults as default_views
from django.views.generic import TemplateView
from django.views.generic.base import RedirectView
from falco_toolbox import views as toolbox_views
from health_check.views import MainView

from .api import api

admin.autodiscover()
admin.site.login = secure_admin_login(admin.site.login)

urlpatterns = [
    path("api/", api.urls),
    path(".well-known/security.txt", toolbox_views.security_txt),
    path("robots.txt", toolbox_views.robots_txt),
    path("", RedirectView.as_view(pattern_name="core:home"), name="home"),
    path(
        "about/",
        TemplateView.as_view(template_name="pages/about.html"),
        name="about",
    ),
    path("health/", MainView.as_view()),
    path(settings.ADMIN_URL, admin.site.urls),
    path("accounts/", include("allauth.urls")),
    path("core/", include("kitchenai.core.urls", namespace="core")),

] + djp.urlpatterns()

if settings.DEBUG:
    urlpatterns += [
        path("__reload__/", include("django_browser_reload.urls")),
        path("__debug__/", include("debug_toolbar.urls")),
        path(
            "400/",
            default_views.bad_request,
            kwargs={"exception": Exception("Bad Request!")},
        ),
        path(
            "403/",
            default_views.permission_denied,
            kwargs={"exception": Exception("Permission Denied")},
        ),
        path(
            "404/",
            default_views.page_not_found,
            kwargs={"exception": Exception("Page not Found")},
        ),
        path("500/", default_views.server_error),
        *static(settings.MEDIA_URL, document_root=settings.MEDIA_ROOT),
    ]



import logging

from django.apps import apps
from django.db.models.signals import post_delete
from django.db.models.signals import post_save
from django.dispatch import receiver
from django_q.tasks import async_task
from kitchenai.contrib.kitchenai_sdk.hooks import delete_file_hook_core, process_file_hook_core
from kitchenai.contrib.kitchenai_sdk.tasks import delete_file_task_core, process_file_task_core, embed_task_core, delete_embed_task_core
import posthog
from .models import FileObject, EmbedObject
logger = logging.getLogger(__name__)


@receiver(post_save, sender=FileObject)
def file_object_created(sender, instance, created, **kwargs):
    """
    This signal is triggered when a new FileObject is created.
    This will trigger any listeners with matching labels and run them as async tasks
    """

    if created:
        #Ninja api should have all bolted on routes and a storage tasks
        logger.info(f"<kitchenai_core>: FileObject created: {instance.pk}")
        posthog.capture("file_object", "kitchenai_file_object_created")

        core_app = apps.get_app_config("core")
        if core_app.kitchenai_app:
            f = core_app.kitchenai_app.storage_tasks(instance.ingest_label)
            if f:
                async_task(process_file_task_core, instance, hook=process_file_hook_core)
            else:
                logger.warning(f"No storage task found for {instance.ingest_label}")
        else:
            logger.warning("module: no kitchenai app found")



@receiver(post_delete, sender=FileObject)
def file_object_deleted(sender, instance, **kwargs):
    """delete the file from vector db"""
    logger.info(f"<kitchenai_core>: FileObject created: {instance.pk}")
    core_app = apps.get_app_config("core")
    if core_app.kitchenai_app:
        f = core_app.kitchenai_app.storage_delete_tasks(instance.ingest_label)
        if f:
            async_task(delete_file_task_core,instance, hook=delete_file_hook_core)
        else:
            logger.warning(f"No storage task found for {instance.ingest_label}")
    else:
        logger.warning("module: no kitchenai app found")

@receiver(post_save, sender=EmbedObject)
def embed_object_created(sender, instance, created, **kwargs):
    """
    This signal is triggered when a new EmbedObject is created.
    This will trigger any listeners with matching labels and run them as async tasks
    """
    if created:
        logger.info(f"<kitchenai_core>: EmbedObject created: {instance.pk}")
        posthog.capture("embed_object", "kitchenai_embed_object_created")

        core_app = apps.get_app_config("core")
        if core_app.kitchenai_app:
            f = core_app.kitchenai_app._embed_tasks.get(f"{core_app.kitchenai_app._namespace}.{instance.ingest_label}")
            print(f"embed_task_core: {core_app.kitchenai_app._embed_tasks}")

            if f:
                #TODO: add hook
                async_task(embed_task_core, instance)
            else:
                logger.warning(f"No embed task found for {instance.ingest_label}")
        else:
            logger.warning("module: no kitchenai app found")

@receiver(post_delete, sender=EmbedObject)
def embed_object_deleted(sender, instance, **kwargs):
    """delete the embed from vector db"""
    logger.info(f"<kitchenai_core>: EmbedObject deleted: {instance.pk}")
    core_app = apps.get_app_config("core")
    if core_app.kitchenai_app:
        f = core_app.kitchenai_app._embed_delete_tasks.get(f"{core_app.kitchenai_app._namespace}.{instance.ingest_label}")
        if f:
            #TODO: add hook
            async_task(delete_embed_task_core, instance)
        else:
            logger.warning(f"No embed delete task found for {instance.ingest_label}")
    else:
        logger.warning("module: no kitchenai app found")



import logging
import os
import sys
from importlib import import_module

import django
import yaml
from django.conf import settings
from django.core.management import call_command
from django.core.management.base import BaseCommand
from kitchenai.api import api
from kitchenai.core.models import KitchenAIManagement

logger = logging.getLogger("kitchenai.core.commands")

class Command(BaseCommand):
    help = 'Runs the development server'


    def handle(self, *args, **options):
        django.setup()
        # Load configuration from the database
        config = self.load_config_from_db()

        if not config:
            self.stdout.write(self.style.ERROR('No configuration found. Please run "kitchenai init" first.'))
            return

        # Update INSTALLED_APPS and import modules
        # self.update_installed_apps(config.get('installed_apps', []))
        self.set_app(config.get("app"))
        # self.import_modules(config.get('module_paths', {}))
        if settings.KITCHENAI_APP:

            # Determine the user's project root directory (assumes the command is run from the user's project root)
            project_root = os.getcwd()

            # Add the user's project root directory to the Python path
            if project_root not in sys.path:
                sys.path.insert(0, project_root)

            print(f"in dynamic routes: {settings.KITCHENAI_APP}")
            module_path, instance_name = settings.KITCHENAI_APP.split(':')
            print(f"module path {module_path}")
            print(f"instance name: {instance_name}")
            try:
                module_path, instance_name = settings.KITCHENAI_APP.split(':')
                module = import_module(module_path)
                instance = getattr(module, instance_name)

                logger.info(f'Imported {instance_name} from {module_path}')
            except (ImportError, AttributeError) as e:
                logger.error(f"Error loading module: {e}")

            print(instance)
            api.add_router("/core", instance)
        call_command('runserver', *args, **options)


    def load_config_from_db(self):
        config = {}
        mgmt = KitchenAIManagement.objects.get(name="kitchenai_management")

        app = mgmt.kitchenaimodules_set.first()
        config["app"] = yaml.safe_load(app.name)
        return config

    def update_installed_apps(self, apps):
        if apps:
            settings.INSTALLED_APPS += tuple(apps)
            self.stdout.write(self.style.SUCCESS(f'Updated INSTALLED_APPS: {settings.INSTALLED_APPS}'))

    def set_app(self, app):
        if app:
            # Set the KITCHENAI_APP setting dynamically
            settings.KITCHENAI_APP = app
            self.stdout.write(self.style.SUCCESS(f'KITCHENAI_APP set to: {settings.KITCHENAI_APP}'))

    def import_modules(self, module_paths):
        for name, path in module_paths.items():
            try:
                module_path, instance_name = path.split(':')
                module = import_module(module_path)
                instance = getattr(module, instance_name)
                globals()[name] = instance
                self.stdout.write(self.style.SUCCESS(f'Imported {instance_name} from {module_path}'))
            except (ImportError, AttributeError) as e:
                self.stdout.write(self.style.ERROR(f"Error loading module '{path}': {e}"))



# Generated by Django 5.1.2 on 2024-11-29 21:24

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("core", "0005_notebook"),
    ]

    operations = [
        migrations.AddField(
            model_name="codefunction",
            name="notebook",
            field=models.ForeignKey(
                blank=True,
                null=True,
                on_delete=django.db.models.deletion.CASCADE,
                to="core.notebook",
            ),
        ),
        migrations.AddField(
            model_name="codeimport",
            name="notebook",
            field=models.ForeignKey(
                blank=True,
                null=True,
                on_delete=django.db.models.deletion.CASCADE,
                to="core.notebook",
            ),
        ),
        migrations.AddField(
            model_name="codesetup",
            name="notebook",
            field=models.ForeignKey(
                blank=True,
                null=True,
                on_delete=django.db.models.deletion.CASCADE,
                to="core.notebook",
            ),
        ),
    ]



# Generated by Django 5.1.2 on 2024-11-29 19:31

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("core", "0001_initial"),
    ]

    operations = [
        migrations.CreateModel(
            name="CodeFunction",
            fields=[
                (
                    "id",
                    models.BigAutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("created_at", models.DateTimeField(auto_now_add=True)),
                ("updated_at", models.DateTimeField(auto_now=True)),
                ("hash", models.CharField(max_length=255)),
                ("raw_code", models.TextField()),
                ("code", models.TextField()),
                (
                    "type",
                    models.CharField(
                        choices=[
                            ("storage", "Storage"),
                            ("embedding", "Embedding"),
                            ("query", "Query"),
                            ("agent", "Agent"),
                        ],
                        max_length=255,
                    ),
                ),
            ],
            options={
                "abstract": False,
            },
        ),
        migrations.CreateModel(
            name="CodeImport",
            fields=[
                (
                    "id",
                    models.BigAutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("created_at", models.DateTimeField(auto_now_add=True)),
                ("updated_at", models.DateTimeField(auto_now=True)),
                ("hash", models.CharField(max_length=255)),
                ("code", models.TextField()),
            ],
            options={
                "abstract": False,
            },
        ),
        migrations.CreateModel(
            name="CodeSetup",
            fields=[
                (
                    "id",
                    models.BigAutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("created_at", models.DateTimeField(auto_now_add=True)),
                ("updated_at", models.DateTimeField(auto_now=True)),
                ("hash", models.CharField(max_length=255)),
                ("code", models.TextField()),
            ],
            options={
                "abstract": False,
            },
        ),
    ]



import os

from django.core.asgi import get_asgi_application

os.environ.setdefault("DJANGO_SETTINGS_MODULE", "kitchenai.settings")

django_asgi = get_asgi_application()

# Starlette serving
from starlette.applications import Starlette
from starlette.routing import Mount

from contextlib import asynccontextmanager
from .broker import broker


@asynccontextmanager
async def broker_lifespan(app):
    await broker.start()
    try:
        yield
    finally:
        await broker.close()


app = Starlette(
    routes=(
        Mount("/", django_asgi),  # redirect all requests to Django
    ),
    # lifespan=broker_lifespan
)
application = app



import typer
from rich.console import Console
from rich.table import Table
from typing_extensions import Annotated
import json
from kitchenai_python_sdk.api_client import ApiClient
from kitchenai_python_sdk.api.default_api import DefaultApi
from kitchenai_python_sdk.configuration import Configuration
from kitchenai_python_sdk.models.embed_schema import EmbedSchema
from kitchenai_python_sdk.models.file_object_schema import FileObjectSchema
from kitchenai_python_sdk.models.query_schema import QuerySchema

app = typer.Typer()
console = Console()

# Configuration for API client
API_HOST = "http://localhost:8001"
configuration = Configuration(host=API_HOST)

file_app = typer.Typer()
embed_app = typer.Typer()

@app.command("health")
def health_check():
    """Check the health of the API."""
    with ApiClient(configuration) as api_client:
        api_instance = DefaultApi(api_client)
        try:
            api_instance.kitchenai_core_api_default()
            console.print("[green]API is healthy![/green]")
        except Exception as e:
            console.print(f"[red]Failed to reach API: {e}[/red]")


@app.command("query")
def run_query(label: str, query: str, metadata: Annotated[str, typer.Option(help="add metadata.")] = ""):
    """Run a query using the Query Handler."""

    if metadata:
        metadata_p = json.loads(metadata)

    else:
        metadata_p = {}

    with ApiClient(configuration) as api_client:
        api_instance = DefaultApi(api_client)
        schema = QuerySchema(query=query, metadata=metadata_p)
        try:
            result = api_instance.kitchenai_core_api_query(label, schema)
            console.print(f"[green]Query '{label}' executed successfully![/green]")
            console.print(result)
        except Exception as e:
            console.print(f"[red]Error running query: {e}[/red]")


@app.command("agent")
def run_agent(label: str, query: str, metadata: Annotated[str, typer.Option(help="add metadata.")] = ""):
    """Run an agent using the Agent Handler."""

    if metadata:
        metadata_p = json.loads(metadata)

    else:
        metadata_p = {}
    with ApiClient(configuration) as api_client:
        api_instance = DefaultApi(api_client)
        schema = QuerySchema(query=query, metadata=metadata_p)
        try:
            api_instance.kitchenai_contrib_kitchenai_sdk_kitchenai_agent_handler(label, schema)
            console.print(f"[green]Agent '{label}' executed successfully![/green]")
        except Exception as e:
            console.print(f"[red]Error running agent: {e}[/red]")


@app.command("file")
def file():
    """File operations."""
    pass

@file_app.command("create")
def create_file(file_path: str, name: str, ingest_label: str, metadata: Annotated[str, typer.Option(help="add metadata.")] = ""):
    """Create a file."""
    if metadata:
        metadata_p = json.loads(metadata)
    else:
        metadata_p = {}

    with open(file_path, 'rb') as file:
        file_content = file.read()
        with ApiClient(configuration) as api_client:
            api_instance = DefaultApi(api_client)
            schema = FileObjectSchema(name=name, ingest_label=ingest_label, metadata=metadata_p)
            try:
                api_response = api_instance.kitchenai_core_api_file_upload(file=(name, file_content), data=schema)
                console.print(f"[green]File '{name}' created successfully! Response: {api_response}[/green]")
            except Exception as e:
                console.print(f"[red]Error creating file: {e}[/red]")

@file_app.command("read")
def read_file(file_id: int):
    """Read a file by ID."""
    with ApiClient(configuration) as api_client:
        api_instance = DefaultApi(api_client)
        try:
            file_data = api_instance.kitchenai_core_api_file_get(file_id)
            console.print(f"[green]File ID '{file_id}' retrieved successfully![/green]")
            console.print(file_data)
        except Exception as e:
            console.print(f"[red]Error reading file: {e}[/red]")

@file_app.command("update")
def update_file(file_id: int, name: str = None, ingest_label: str = None, metadata: Annotated[str, typer.Option(help="add metadata.")] = ""):
    """Update a file by ID."""
    if metadata:
        metadata_p = json.loads(metadata)
    else:
        metadata_p = {}

    with ApiClient(configuration) as api_client:
        api_instance = DefaultApi(api_client)
        schema = FileObjectSchema(name=name, ingest_label=ingest_label, metadata=metadata_p)
        try:
            api_response = api_instance.kitchenai_core_api_file_update(file_id, data=schema)
            console.print(f"[green]File ID '{file_id}' updated successfully! Response: {api_response}[/green]")
        except Exception as e:
            console.print(f"[red]Error updating file: {e}[/red]")

@file_app.command("delete")
def delete_file(file_id: int):
    """Delete a file by ID."""
    with ApiClient(configuration) as api_client:
        api_instance = DefaultApi(api_client)
        try:
            api_instance.kitchenai_core_api_file_delete(file_id)
            console.print(f"[green]File ID '{file_id}' deleted successfully![/green]")
        except Exception as e:
            console.print(f"[red]Error deleting file: {e}[/red]")

@file_app.command("list")
def list_files():
    """List all files."""
    with ApiClient(configuration) as api_client:
        api_instance = DefaultApi(api_client)
        try:
            files = api_instance.kitchenai_core_api_files_get()
            if not files:
                console.print("[yellow]No files found.[/yellow]")
                return
            table = Table(title="Files")
            table.add_column("ID", style="cyan", no_wrap=True)
            table.add_column("Name", style="magenta")
            table.add_column("Ingest Label", style="green")
            for file in files:
                table.add_row(str(file.id), file.name, file.ingest_label or "N/A")
            console.print(table)
        except Exception as e:
            console.print(f"[red]Error fetching files: {e}[/red]")

app.add_typer(file_app, name="file")

@app.command("embed")
def embed():
    """Embed operations."""
    pass

@embed_app.command("list")
def get_all_embeds():
    """Get all embeds."""
    with ApiClient(configuration) as api_client:
        api_instance = DefaultApi(api_client)
        try:
            embeds = api_instance.kitchenai_core_api_embeds_get()
            if not embeds:
                console.print("[yellow]No embeds found.[/yellow]")
                return
            table = Table(title="Embeds")
            table.add_column("ID", style="cyan", no_wrap=True)
            table.add_column("Text", style="magenta")
            for embed in embeds:
                table.add_row(str(embed.id), embed.text)
            console.print(table)
        except Exception as e:
            console.print(f"[red]Error fetching embeds: {e}[/red]")

@embed_app.command("create")
def create_embed(text: str, ingest_label: str, metadata: Annotated[str, typer.Option(help="add metadata.")] = ""):
    """Create an embed."""
    if metadata:
        metadata_p = json.loads(metadata)
    else:
        metadata_p = {}
    with ApiClient(configuration) as api_client:
        api_instance = DefaultApi(api_client)
        schema = EmbedSchema(text=text, ingest_label=ingest_label, metadata=metadata_p)
        try:
            response = api_instance.kitchenai_core_api_embed_create(schema)
            console.print(f"[green]Embed created successfully! Response: {response}[/green]")
        except Exception as e:
            console.print(f"[red]Error creating embed: {e}[/red]")

@embed_app.command("delete")
def delete_embed(embed_id: int):
    """Delete an embed by ID."""
    with ApiClient(configuration) as api_client:
        api_instance = DefaultApi(api_client)
        try:
            api_instance.kitchenai_core_api_embed_delete(embed_id)
            console.print(f"[green]Embed ID '{embed_id}' deleted successfully![/green]")
        except Exception as e:
            console.print(f"[red]Error deleting embed: {e}[/red]")

app.add_typer(embed_app, name="embed")

@app.command("labels")
def list_labels():
    """List all custom kitchenai labels."""
    with ApiClient(configuration) as api_client:
        api_instance = DefaultApi(api_client)
        try:
            labels = api_instance.kitchenai_core_api_labels()
            if not labels:
                console.print("[yellow]No labels found.[/yellow]")
                return
            table = Table(title="Labels")
            table.add_column("Namespace", style="cyan", no_wrap=True)
            table.add_column("Query Handlers", style="magenta")
            table.add_column("Agent Handlers", style="green")
            table.add_column("Embed Tasks", style="blue")
            table.add_column("Embed Delete Tasks", style="red")
            table.add_column("Storage Tasks", style="yellow")
            table.add_column("Storage Delete Tasks", style="purple")
            table.add_column("Storage Create Hooks", style="white")
            table.add_column("Storage Delete Hooks", style="blue")

            table.add_row(
                labels.namespace,
                ", ".join(labels.query_handlers),
                ", ".join(labels.agent_handlers),
                ", ".join(labels.embed_tasks),
                ", ".join(labels.embed_delete_tasks),
                ", ".join(labels.storage_tasks),
                ", ".join(labels.storage_delete_tasks),
                ", ".join(labels.storage_create_hooks),
                ", ".join(labels.storage_delete_hooks),
            )
            console.print(table)
        except Exception as e:
            console.print(f"[red]Error fetching labels: {e}[/red]")





# Generated by Django 5.1.2 on 2024-11-29 21:20

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("core", "0004_alter_codefunction_label"),
    ]

    operations = [
        migrations.CreateModel(
            name="Notebook",
            fields=[
                (
                    "id",
                    models.BigAutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("created_at", models.DateTimeField(auto_now_add=True)),
                ("updated_at", models.DateTimeField(auto_now=True)),
                ("name", models.CharField(max_length=255, unique=True)),
            ],
            options={
                "abstract": False,
            },
        ),
    ]



from django.urls import path

from . import views

app_name = "core"

urlpatterns = [
    path("", views.home, name="home"),
]

urlpatterns += [
    path(
        "kitchen-ai-modules/", views.kitchenaimodule_list, name="kitchenaimodule_list"
    ),
    path(
        "kitchen-ai-modules/create/",
        views.kitchenaimodule_create,
        name="kitchenaimodule_create",
    ),
    path(
        "kitchen-ai-modules/<int:pk>/",
        views.kitchenaimodule_detail,
        name="kitchenaimodule_detail",
    ),
    path(
        "kitchen-ai-modules/<int:pk>/update/",
        views.kitchenaimodule_update,
        name="kitchenaimodule_update",
    ),
    path(
        "kitchen-ai-modules/<int:pk>/delete/",
        views.kitchenaimodule_delete,
        name="kitchenaimodule_delete",
    ),
]



import logging
import pathlib

import yaml
from django.core.management.base import BaseCommand
from django.template import loader


logger = logging.getLogger("kitchenai.core.commands")


class Command(BaseCommand):
    help = 'Builds kitchenai'

    def handle(self, *args, **options):
        """Checks to see if there is a kitchen.yml in the root directory, loads and initializes the db"""

        config = pathlib.Path.cwd() / "kitchenai.yml"
        config_data = self.read_yaml_config(config)

        if config_data:
            # Save the configuration to the database
            template_name = 'build_templates/Dockerfile.tmpl'

            # Context data to pass into the template
            context = {
                'title': 'Django Template Rendering',
                'message': 'This is rendered via a custom management command!'
            }

            try:
                # Load the template
                template = loader.get_template(template_name)

                # Render the template with the context data
                rendered_content = template.render(context)

                # Output to standard output
                self.stdout.write(self.style.SUCCESS(rendered_content))

            except Exception as e:
                self.stderr.write(self.style.ERROR(f"Error rendering template: {e}"))
        else:
            self.stdout.write(self.style.ERROR('No config file for kitchenai.'))

    def read_yaml_config(self, yaml_file_path):

        # Check if the file exists
        if not yaml_file_path.is_file():
            self.stdout.write(self.style.ERROR("YAML config file kitchenai.yml not found."))
            return None

        # Read the YAML file
        with open(yaml_file_path) as file:
            try:
                return yaml.safe_load(file)
            except yaml.YAMLError as e:
                self.stdout.write(self.style.ERROR(f"Error reading YAML file: {e}"))
                return None



from unittest.mock import Mock
from unittest.mock import patch

import pytest
from django.core.files.uploadedfile import SimpleUploadedFile
from django.test import Client


@pytest.fixture
def api_client():
    return Client()

@pytest.fixture
def mock_chroma_client():
    with patch('chromadb.EphemeralClient') as mock:
        yield mock

@pytest.fixture
def mock_llm():
    with patch('llama_index.llms.openai.OpenAI') as mock:
        yield mock

class TestAIClient:

    @pytest.mark.asyncio
    @patch('llama_index.core.VectorStoreIndex.from_documents')
    @patch('llama_index.core.SimpleDirectoryReader')
    async def test_storage_endpoint(self, mock_reader, mock_index, api_client):
        # Create a mock file
        file_content = b"This is a test document"
        uploaded_file = SimpleUploadedFile(
            "test.txt",
            file_content,
            content_type="text/plain"
        )

        # Mock the SimpleDirectoryReader
        mock_documents = [Mock()]
        mock_reader.return_value.load_data.return_value = mock_documents

        # Make the request
        response = api_client.post(
            "/api/storage",
            {"file": uploaded_file},
            format="multipart"
        )

        # Assertions
        assert response.status_code == 200
        assert response.json() == {"msg": "ok"}

        # Verify that the vector store was properly called
        mock_index.assert_called_once()

    @pytest.mark.asyncio
    @patch('llama_index.core.VectorStoreIndex.from_vector_store')
    async def test_query_endpoint(self, mock_index, api_client, mock_llm):
        # Mock the chat engine response
        mock_chat_engine = Mock()
        mock_response = Mock()
        mock_response.response = "This is a test response"
        mock_chat_engine.achat.return_value = mock_response

        mock_index.return_value.as_chat_engine.return_value = mock_chat_engine

        # Make the request
        query_data = {"query": "What is the meaning of life?"}
        response = api_client.post(
            "/api/query",
            query_data,
            content_type="application/json"
        )

        # Assertions
        assert response.status_code == 200
        assert response.json() == {"msg": "This is a test response"}

        # Verify that the chat engine was called with correct parameters
        mock_chat_engine.achat.assert_called_once_with(query_data["query"])

    @pytest.mark.asyncio
    async def test_query_endpoint_invalid_input(self, api_client):
        # Test with missing query field
        response = api_client.post(
            "/api/query",
            {},
            content_type="application/json"
        )
        assert response.status_code == 422  # Validation error

    @pytest.mark.asyncio
    @patch('llama_index.core.VectorStoreIndex.from_vector_store')
    async def test_query_endpoint_error_handling(self, mock_index, api_client):
        # Mock an error in the chat engine
        mock_chat_engine = Mock()
        mock_chat_engine.achat.side_effect = Exception("Test error")
        mock_index.return_value.as_chat_engine.return_value = mock_chat_engine

        # Make the request
        query_data = {"query": "What is the meaning of life?"}
        response = api_client.post(
            "/api/query",
            query_data,
            content_type="application/json"
        )

        # Assertions
        assert response.status_code == 500



import uuid

from django.db import models
from falco_toolbox.models import TimeStamped

def file_object_directory_path(instance, filename):
    # file will be uploaded to MEDIA_ROOT/uuid/filename
    return f"kitchenai/{uuid.uuid4()}/{filename}"

def module_directory_path(instance, filename):
    # file will be uploaded to MEDIA_ROOT/uuid/filename
    return f"kitchenai/modules/{filename}"

class KitchenAIManagement(TimeStamped):
    name = models.CharField(max_length=255, primary_key=True, default="kitchenai_management")
    project_name = models.CharField(max_length=255)
    version = models.CharField(max_length=255)
    description = models.TextField(default="")
    jupyter_token = models.CharField(max_length=255, default="")
    jupyter_host = models.CharField(max_length=255, default="")
    jupyter_port = models.CharField(max_length=255, default="8888")
    jupyter_protocol = models.CharField(max_length=255, default="http")

    def __str__(self):
        return self.name


class KitchenAIPlugins(TimeStamped):
    name = models.CharField(max_length=255, unique=True)
    kitchen = models.ForeignKey(KitchenAIManagement, on_delete=models.CASCADE)

    def __str__(self):
        return self.name


class KitchenAIDependencies(TimeStamped):
    name = models.CharField(max_length=255, unique=True)
    kitchen = models.ForeignKey(KitchenAIManagement, on_delete=models.CASCADE)

    def __str__(self):
        return self.name


class KitchenAIRootModule(TimeStamped):
    name = models.CharField(max_length=255, unique=True)
    kitchen = models.ForeignKey(KitchenAIManagement, on_delete=models.CASCADE)

class KitchenAIModule(TimeStamped):
    name = models.CharField(max_length=255)
    kitchen = models.ForeignKey(KitchenAIManagement, on_delete=models.CASCADE)
    jupyter_path = models.CharField(max_length=255, default="")
    file = models.FileField(upload_to=module_directory_path)


class Notebook(TimeStamped):
    name = models.CharField(max_length=255, unique=True)

    def __str__(self) -> str:
        return self.name

class CodeFunction(TimeStamped):
    class FuncType(models.TextChoices):
        STORAGE = "storage"
        EMBEDDING = "embedding"
        QUERY = "query"
        AGENT = "agent"

    hash = models.CharField(max_length=255)
    raw_code = models.TextField()
    code = models.TextField()
    type = models.CharField(max_length=255, choices=FuncType)
    label = models.CharField(max_length=255)
    notebook =  models.ForeignKey(Notebook, on_delete=models.CASCADE, blank=True, null=True)

    def __str__(self) -> str:
        return self.label

class CodeImport(TimeStamped):
    hash = models.CharField(max_length=255)
    code = models.TextField()
    notebook =  models.ForeignKey(Notebook, on_delete=models.CASCADE, blank=True, null=True)
    label =  models.CharField(max_length=255)

    def __str__(self) -> str:
        return f"<notebook: {self.notebook}: {self.hash}>"
    
class CodeSetup(TimeStamped):
    hash = models.CharField(max_length=255)
    code = models.TextField()
    notebook =  models.ForeignKey(Notebook, on_delete=models.CASCADE, blank=True, null=True)
    label = models.CharField(max_length=255)


    def __str__(self) -> str:
        return f"<notebook: {self.notebook}: {self.hash}>"



class FileObject(TimeStamped):
    """
    This is a model for any file that is uploaded to the system.
    It will be used to trigger any storage tasks or other processes
    """
    class Status(models.TextChoices):
        PENDING = "pending"
        PROCESSING = "processing"
        COMPLETED = "completed"
        FAILED = "failed"

    file = models.FileField(upload_to=file_object_directory_path)
    name = models.CharField(max_length=255)
    ingest_label = models.CharField(max_length=255)
    status = models.CharField(max_length=255, default=Status.PENDING)
    metadata = models.JSONField(default=dict)

    def __str__(self):
        return self.name
    
class EmbedObject(TimeStamped):
    """
    This is a model for any embed object that is created
    """
    class Status(models.TextChoices):
        PENDING = "pending"
        PROCESSING = "processing"
        COMPLETED = "completed"
        FAILED = "failed"

    text = models.CharField(max_length=255)
    ingest_label = models.CharField(max_length=255)
    status = models.CharField(max_length=255, default=Status.PENDING)
    metadata = models.JSONField(default=dict)

    def __str__(self):
        return self.text




from django import forms

from .models import KitchenAIModule


class KitchenAIModuleForm(forms.ModelForm):
    class Meta:
        model = KitchenAIModule
        fields = ("name", "kitchen", "jupyter_path", "file")



from pathlib import Path

import requests
import typer
from rich.console import Console
from rich.table import Table


app = typer.Typer()
console = Console()



API_BASE_URL = "https://raw.githubusercontent.com/epuerta9/kitchenai-community/main"

@app.command("list")
def cook_list():
    """List all available starter cookbooks."""
    response = requests.get(f"{API_BASE_URL}/cookbooks.json")
    if response.status_code != 200:
        console.print("[red]Error fetching cookbook list.[/red]")
        raise typer.Exit(code=1)

    cookbooks = response.json().get("cookbooks", [])
    if not cookbooks:
        console.print("[yellow]No cookbooks available.[/yellow]")
        return

    # Create a table for the cookbooks
    table = Table(title="Available Starter Cookbooks")
    table.add_column("Name", style="cyan", no_wrap=True)
    table.add_column("Description", style="magenta")

    for cookbook in cookbooks:
        table.add_row(cookbook["name"], cookbook["description"])

    # Display the table
    console.print(table)


@app.command("select")
def cook_select(name: str):
    """Download a specific starter cookbook."""
    response = requests.get(f"{API_BASE_URL}/cookbooks.json")
    if response.status_code != 200:
        console.print("[red]Error fetching cookbook list.[/red]")
        raise typer.Exit(code=1)

    cookbooks = response.json().get("cookbooks", [])
    selected_cookbook = next((c for c in cookbooks if c["name"] == name), None)

    if not selected_cookbook:
        console.print("[red]Cookbook not found.[/red]")
        raise typer.Exit(code=1)

    files_to_download = ["notebook.ipynb", "app.py", "README.md", "requirements.txt"]
    dest_path = Path.cwd()

    # Show spinner while downloading the files
    with console.status(f"[cyan]Downloading {name} cookbook...[/cyan]", spinner="dots"):
        for file in files_to_download:
            file_url = f"{API_BASE_URL}/{selected_cookbook['path']}/{file}"
            file_response = requests.get(file_url)

            if file_response.status_code == 200:
                with open(dest_path / file, "wb") as f:
                    f.write(file_response.content)
                console.print(f"[green]Downloaded {file} successfully.[/green]")
            else:
                console.print(f"[red]Error downloading {file}.[/red]")
                raise typer.Exit(code=1)



import multiprocessing
import os
import sys
from email.utils import parseaddr
from pathlib import Path

import djp
import sentry_sdk
from environs import Env
from falco_toolbox.sentry import sentry_profiles_sampler
from falco_toolbox.sentry import sentry_traces_sampler
from marshmallow.validate import Email
from marshmallow.validate import OneOf
from sentry_sdk.integrations.django import DjangoIntegration
from sentry_sdk.integrations.logging import LoggingIntegration

# 0. Setup
# --------------------------------------------------------------------------------------------


VERSION = "0.0.1"

BASE_DIR = Path(__file__).resolve(strict=True).parent.parent

APPS_DIR = BASE_DIR / "kitchenai"

KITCHENAI_DB_DIR = BASE_DIR / ".kitchenai"

KITCHENAI_DB_DIR.mkdir(exist_ok=True)

env = Env()
env.read_env(Path(BASE_DIR, ".env").as_posix())

# We should strive to only have two possible runtime scenarios: either `DEBUG`
# is True or it is False. `DEBUG` should be only true in development, and
# False when deployed, whether or not it's a production environment.
DEBUG = env.bool("DEBUG", default=False)
KITCHENAI_DEBUG = env.bool("KITCHENAI_DEBUG", default=False)

# 1. Django Core Settings
# -----------------------------------------------------------------------------------------------
# https://docs.djangoproject.com/en/4.0/ref/settings/


ALLOWED_HOSTS = env.list("ALLOWED_HOSTS", default=["*"] if DEBUG or KITCHENAI_DEBUG else ["localhost"], subcast=str)

ASGI_APPLICATION = "kitchenai.asgi.application"

# https://grantjenks.com/docs/diskcache/tutorial.html#djangocache
if "CACHE_LOCATION" in os.environ:
    CACHES = {
        "default": {
            "BACKEND": "diskcache.DjangoCache",
            "LOCATION": env.str("CACHE_LOCATION"),
            "TIMEOUT": 300,
            "SHARDS": 8,
            "DATABASE_TIMEOUT": 0.010,  # 10 milliseconds
            "OPTIONS": {"size_limit": 2**30},  # 1 gigabyte
        }
    }

CSRF_COOKIE_SECURE = not DEBUG

DATABASES = {
    "default": env.dj_db_url("DATABASE_URL", default=f"sqlite:///{KITCHENAI_DB_DIR / 'db.sqlite3'}"),
}
DATABASES["default"]["ATOMIC_REQUESTS"] = False

if not DEBUG or KITCHENAI_DEBUG:
    DATABASES["default"]["CONN_MAX_AGE"] = env.int("CONN_MAX_AGE", default=60)
    DATABASES["default"]["CONN_HEALTH_CHECKS"] = True

DEFAULT_AUTO_FIELD = "django.db.models.BigAutoField"

DEFAULT_FROM_EMAIL = env.str(
    "DEFAULT_FROM_EMAIL",
    default="example@example.com",
    validate=lambda v: Email()(parseaddr(v)[1]),
)

EMAIL_BACKEND = (
    "django.core.mail.backends.console.EmailBackend"
    if (DEBUG or KITCHENAI_DEBUG)
    else "anymail.backends.resend.EmailBackend"
)

DJANGO_APPS = [
    "django.contrib.admin",
    "django.contrib.auth",
    "django.contrib.contenttypes",
    "django.contrib.sessions",
    "django.contrib.messages",
    "django.contrib.staticfiles",
    "django.contrib.humanize",
    "django.forms",
]

THIRD_PARTY_APPS = [
    "allauth_ui",
    "allauth",
    "allauth.account",
    "allauth.socialaccount",
    "compressor",
    "crispy_forms",
    "crispy_tailwind",
    "django_extensions",
    "django_htmx",
    "django_q",
    "django_q_registry",
    "django_tailwind_cli",
    "falco_toolbox",
    "health_check",
    "health_check.cache",
    "health_check.contrib.migrations",
    "health_check.db",
    "health_check.storage",
    "heroicons",
    "template_partials",
    "unique_user_email",
    "widget_tweaks",
    "slippers",
]

LOCAL_APPS = [
    "kitchenai.core",
]

if DEBUG:
    # Development only apps
    THIRD_PARTY_APPS = [
        "debug_toolbar",
        "whitenoise.runserver_nostatic",
        "django_browser_reload",
        "django_fastdev",
        "django_watchfiles",
        *THIRD_PARTY_APPS,
    ]

INSTALLED_APPS = LOCAL_APPS + THIRD_PARTY_APPS + DJANGO_APPS

if DEBUG or KITCHENAI_DEBUG:
    INTERNAL_IPS = [
        "127.0.0.1",
        "10.0.2.2",
    ]

LANGUAGE_CODE = "en-us"

# LOGGING = {
#     "version": 1,
#     "disable_existing_loggers": False,
#     "formatters": {
#         "plain_console": {
#             "format": "%(levelname)s %(message)s",
#         },
#         "verbose": {
#             "format": "%(asctime)s %(name)-12s %(levelname)-8s %(message)s",
#         },
#     },
#     "handlers": {
#         "stdout": {
#             "class": "logging.StreamHandler",
#             "stream": sys.stdout,
#             # "formatter": "verbose",
#         },
#     },
#     "loggers": {
#         "django": {
#             "handlers": ["stdout"],
#             "level": env.log_level("DJANGO_LOG_LEVEL", default="INFO"),
#         },
#         "kitchenai": {
#             "handlers": ["stdout"],
#             "level": env.log_level("KITCHENAI_LOG_LEVEL", default="INFO"),
#         },
#     },
# }

LOGGING = {
    'version': 1,
    'disable_existing_loggers': False,
    'formatters': {
        'verbose': {
            'format': '{levelname} {asctime} {module} {message}',
            'style': '{',
        },
        'simple': {
            'format': '{levelname} {message}',
            'style': '{',
        },
    },
    'handlers': {
        'console': {
            'class': 'logging.StreamHandler',
            'formatter': 'simple',
        },
    },
    'loggers': {
        'django': {
            'handlers': ['console'],
            'level': 'INFO',  # Set to INFO or WARNING to suppress DEBUG logs
            'propagate': True,
        },
        'urllib3.connectionpool': {
            'handlers': ['console'],
            'level': 'WARNING',  # Suppress DEBUG logs from urllib3
            'propagate': False,
        },
        'chromadb': {
            'handlers': ['console'],
            'level': 'WARNING',  # Suppress DEBUG logs from chromadb
            'propagate': False,
        },
        "kitchenai": {
            "handlers": ["console"],
            "level": env.log_level("KITCHENAI_LOG_LEVEL", default="DEBUG"),
            "propagate": False,  # Prevent propagation to the root logger
        },
    },
    'root': {
        'handlers': ['console'],
        'level': 'INFO',  # Set the root logger level
    },
}

MEDIA_ROOT = env.path("MEDIA_ROOT", default=APPS_DIR / "media")

MEDIA_URL = "/media/"

# https://docs.djangoproject.com/en/dev/topics/http/middleware/
# https://docs.djangoproject.com/en/dev/ref/middleware/#middleware-ordering
MIDDLEWARE = [
    # should be first
    "django.middleware.cache.UpdateCacheMiddleware",
    "django.middleware.security.SecurityMiddleware",
    "whitenoise.middleware.WhiteNoiseMiddleware",
    # order doesn't matter
    "django.contrib.sessions.middleware.SessionMiddleware",
    "django.middleware.common.CommonMiddleware",
    "django.middleware.csrf.CsrfViewMiddleware",
    "django.contrib.auth.middleware.AuthenticationMiddleware",
    "django.contrib.messages.middleware.MessageMiddleware",
    "django.middleware.clickjacking.XFrameOptionsMiddleware",
    "allauth.account.middleware.AccountMiddleware",
    "django_htmx.middleware.HtmxMiddleware",
    # should be last
    "django.middleware.cache.FetchFromCacheMiddleware",
]

if DEBUG or KITCHENAI_DEBUG:
    MIDDLEWARE.remove("django.middleware.cache.UpdateCacheMiddleware")
    MIDDLEWARE.remove("django.middleware.cache.FetchFromCacheMiddleware")

if DEBUG:
    MIDDLEWARE.append("django_browser_reload.middleware.BrowserReloadMiddleware")

    MIDDLEWARE.insert(
        MIDDLEWARE.index("django.middleware.common.CommonMiddleware") + 1,
        "debug_toolbar.middleware.DebugToolbarMiddleware",
    )

ROOT_URLCONF = "kitchenai.urls"

SECRET_KEY = env.str("SECRET_KEY", default="django-insecure-ef6nIh7LcUjPtixFdz0_aXyUwlKqvBdJEcycRR6RvRY")

SECURE_HSTS_INCLUDE_SUBDOMAINS = not (DEBUG or KITCHENAI_DEBUG)

SECURE_HSTS_PRELOAD = not (DEBUG or KITCHENAI_DEBUG)

# https://docs.djangoproject.com/en/dev/ref/middleware/#http-strict-transport-security
# 2 minutes to start with, will increase as HSTS is tested
# example of production value: 60 * 60 * 24 * 7 = 604800 (1 week)
SECURE_HSTS_SECONDS = 0 if DEBUG or KITCHENAI_DEBUG else env.int("SECURE_HSTS_SECONDS", default=60 * 2)

# https://noumenal.es/notes/til/django/csrf-trusted-origins/
# SECURE_PROXY_SSL_HEADER = ("HTTP_X_FORWARDED_PROTO", "https")

# SECURE_SSL_REDIRECT = not DEBUG

SERVER_EMAIL = env.str(
    "SERVER_EMAIL",
    default=DEFAULT_FROM_EMAIL,
    validate=lambda v: Email()(parseaddr(v)[1]),
)

SESSION_COOKIE_SECURE = not (DEBUG or KITCHENAI_DEBUG)

STORAGES = {
    "default": {
        "BACKEND": "storages.backends.s3.S3Storage",
        "OPTIONS": {
            "access_key": env.str("AWS_ACCESS_KEY_ID", default=None),
            "bucket_name": env.str("AWS_STORAGE_BUCKET_NAME", default=None),
            "region_name": env.str("AWS_S3_REGION_NAME", default=None),
            "secret_key": env.str("AWS_SECRET_ACCESS_KEY", default=None),
        },
    },
    "staticfiles": {
        "BACKEND": "whitenoise.storage.CompressedManifestStaticFilesStorage",
    },
}
if (DEBUG or KITCHENAI_DEBUG) and not env.bool("USE_S3", default=False):
    STORAGES["default"] = {
        "BACKEND": "django.core.files.storage.FileSystemStorage",
    }

# https://nickjanetakis.com/blog/django-4-1-html-templates-are-cached-by-default-with-debug-true
DEFAULT_LOADERS = [
    "django.template.loaders.filesystem.Loader",
    "django.template.loaders.app_directories.Loader",
]

CACHED_LOADERS = [("django.template.loaders.cached.Loader", DEFAULT_LOADERS)]

TEMPLATES = [
    {
        "BACKEND": "django.template.backends.django.DjangoTemplates",
        "DIRS": [str(APPS_DIR / "templates")],
        "OPTIONS": {
            "context_processors": [
                "django.template.context_processors.debug",
                "django.template.context_processors.request",
                "django.contrib.auth.context_processors.auth",
                "django.contrib.messages.context_processors.messages",
            ],
            "builtins": [
                "template_partials.templatetags.partials",
                "heroicons.templatetags.heroicons",
            ],
            "debug": DEBUG,
            "loaders": [
                (
                    "template_partials.loader.Loader",
                    DEFAULT_LOADERS if (DEBUG or KITCHENAI_DEBUG) else CACHED_LOADERS,
                )
            ],
        },
    },
]

TIME_ZONE = "UTC"

USE_I18N = False

USE_TZ = True

WSGI_APPLICATION = "kitchenai.wsgi.application"

# 2. Django Contrib Settings
# -----------------------------------------------------------------------------------------------

# django.contrib.auth
AUTHENTICATION_BACKENDS = [
    "django.contrib.auth.backends.ModelBackend",
    "allauth.account.auth_backends.AuthenticationBackend",
]

AUTH_PASSWORD_VALIDATORS = [
    {
        "NAME": "django.contrib.auth.password_validation.UserAttributeSimilarityValidator",
    },
    {
        "NAME": "django.contrib.auth.password_validation.MinimumLengthValidator",
    },
    {
        "NAME": "django.contrib.auth.password_validation.CommonPasswordValidator",
    },
    {
        "NAME": "django.contrib.auth.password_validation.NumericPasswordValidator",
    },
]
if DEBUG or KITCHENAI_DEBUG:
    AUTH_PASSWORD_VALIDATORS = []

# django.contrib.staticfiles
STATIC_ROOT = APPS_DIR / "staticfiles"

STATIC_URL = "/static/"

STATICFILES_DIRS = [APPS_DIR / "static"]

STATICFILES_FINDERS = (
    "django.contrib.staticfiles.finders.FileSystemFinder",
    "django.contrib.staticfiles.finders.AppDirectoriesFinder",
    "compressor.finders.CompressorFinder",
)

# 3. Third Party Settings
# -------------------------------------------------------------------------------------------------

# django-allauth
ACCOUNT_AUTHENTICATION_METHOD = "email"

ACCOUNT_DEFAULT_HTTP_PROTOCOL = "http" if (DEBUG or KITCHENAI_DEBUG) else "https"

ACCOUNT_EMAIL_REQUIRED = True

ACCOUNT_LOGOUT_REDIRECT_URL = "account_login"

ACCOUNT_SESSION_REMEMBER = True

ACCOUNT_SIGNUP_PASSWORD_ENTER_TWICE = False

ACCOUNT_UNIQUE_EMAIL = True

ACCOUNT_USERNAME_REQUIRED = False

LOGIN_REDIRECT_URL = "home"

# django-anymail
if not (DEBUG or KITCHENAI_DEBUG):
    resend_api_key = env.str("RESEND_API_KEY", default=None)
    if resend_api_key:
        ANYMAIL = {
            "RESEND_API_KEY": resend_api_key,
        }
    else:
        EMAIL_BACKEND = "django.core.mail.backends.console.EmailBackend"

# django-compressor
COMPRESS_ENABLED = not (DEBUG or KITCHENAI_DEBUG)
COMPRESS_OFFLINE = not (DEBUG or KITCHENAI_DEBUG)
COMPRESS_FILTERS = {
    "css": [
        "compressor.filters.css_default.CssAbsoluteFilter",
        "compressor.filters.cssmin.rCSSMinFilter",
        "refreshcss.filters.RefreshCSSFilter",
    ],
    "js": ["compressor.filters.jsmin.rJSMinFilter"],
}

# django-crispy-forms
CRISPY_ALLOWED_TEMPLATE_PACKS = "tailwind"

CRISPY_TEMPLATE_PACK = "tailwind"

# django-debug-toolbar
DEBUG_TOOLBAR_CONFIG = {
    "DISABLE_PANELS": ["debug_toolbar.panels.redirects.RedirectsPanel"],
    "SHOW_TEMPLATE_CONTEXT": True,
    "SHOW_COLLAPSED": True,
    "UPDATE_ON_FETCH": True,
    "ROOT_TAG_EXTRA_ATTRS": "hx-preserve",
}

# django-q2
Q_CLUSTER = {
    "name": "ORM",
    "workers": multiprocessing.cpu_count() * 2 + 1,
    "timeout": 60 * 10,  # 10 minutes
    "retry": 60 * 12,  # 12 minutes
    "queue_limit": 50,
    "bulk": 10,
    "orm": "default",
}

# sentry
if (SENTRY_DSN := env.url("SENTRY_DSN", default=None)).scheme and not (DEBUG or KITCHENAI_DEBUG):
    sentry_sdk.init(
        dsn=SENTRY_DSN.geturl(),
        environment=env.str(
            "SENTRY_ENV",
            default="development",
            validate=OneOf(["development", "production"]),
        ),
        integrations=[
            DjangoIntegration(),
            LoggingIntegration(event_level=None, level=None),
        ],
        traces_sampler=sentry_traces_sampler,
        profiles_sampler=sentry_profiles_sampler,
        send_default_pii=True,
    )

# 4. Project Settings
# -----------------------------------------------------------------------------------------------------

ADMIN_URL = env.str("ADMIN_URL", default="kitchenai-admin/")


# KITCHEN AI

KITCHENAI = {}


# Django plugin system. This has to be the last line
djp.settings(globals())



from django.http import HttpRequest, HttpResponse
from django.shortcuts import get_object_or_404, redirect
from django.template.response import TemplateResponse
from django.views.decorators.http import require_http_methods
from falco_toolbox.htmx import for_htmx
from falco_toolbox.pagination import paginate_queryset
from falco_toolbox.types import HttpRequest

from .forms import KitchenAIModuleForm
from .models import KitchenAIModule


async def home(request: HttpRequest):
    return TemplateResponse(
        request,
        "pages/home.html",
    )


@for_htmx(use_partial="table")
def kitchenaimodule_list(request: HttpRequest):
    kitchenaimodules = KitchenAIModule.objects.order_by("name")
    return TemplateResponse(
        request,
        "core/kitchenaimodule_list.html",
        context={"kitchenaimodules_page": paginate_queryset(request, kitchenaimodules)},
    )


def kitchenaimodule_detail(request: HttpRequest, pk: int):
    kitchenaimodule = get_object_or_404(KitchenAIModule.objects, pk=pk)
    return TemplateResponse(
        request,
        "core/kitchenaimodule_detail.html",
        context={"kitchenaimodule": kitchenaimodule},
    )


def kitchenaimodule_create(request: HttpRequest):
    form = KitchenAIModuleForm(request.POST or None, request.FILES or None)
    if request.method == "POST" and form.is_valid():
        form.save()
        return redirect("core:kitchenaimodule_list")
    return TemplateResponse(
        request,
        "core/kitchenaimodule_create.html",
        context={"form": form},
    )


def kitchenaimodule_update(request: HttpRequest, pk: int):
    kitchenaimodule = get_object_or_404(KitchenAIModule.objects, pk=pk)
    form = KitchenAIModuleForm(
        request.POST or None, request.FILES or None, instance=kitchenaimodule
    )
    if request.method == "POST" and form.is_valid():
        form.save()
        return redirect("core:kitchenaimodule_detail", pk=pk)
    return TemplateResponse(
        request,
        "core/kitchenaimodule_update.html",
        context={"kitchenaimodule": kitchenaimodule, "form": form},
    )


@require_http_methods(["DELETE"])
def kitchenaimodule_delete(_: HttpRequest, pk: int):
    KitchenAIModule.objects.filter(pk=pk).delete()
    return HttpResponse("")



import logging
import os
import sys

import django
import typer
from cookiecutter.main import cookiecutter
from django.conf import settings
from rich.console import Console
from typing import Annotated

from .cook import app as cook_app
from .client import app as client_app

console = Console()

logger = logging.getLogger(__name__)

app = typer.Typer()
app.add_typer(cook_app, name="cook")
app.add_typer(client_app, name="client")


@app.command()
def add(module: str = typer.Argument("app.kitchen:kitchen")):
    from django.core.management import execute_from_command_line

    execute_from_command_line(["manage", "add_module", module])

@app.command()
def init(
    verbose: Annotated[int, typer.Option(help="verbosity level. default 0")] = 0,
    collect_static: Annotated[bool, typer.Option("--collect-static/--no-collect-static", help="Collect static assets.")] = False,
    ):
    django.setup()
    from django.core.management import execute_from_command_line
    from kitchenai.core.models import KitchenAIManagement
    from django.conf import settings
    import posthog

    posthog.capture("init", "kitchenai_init")

    cmd = ["manage", "migrate","--verbosity", f"{verbose}"]

    if verbose != 1:
        with console.status("Applying migrations...", spinner="dots"):
            execute_from_command_line(cmd)

        with console.status("Setting up periodic tasks", spinner="dots"):
            execute_from_command_line(["manage", "setup_periodic_tasks"])

        if collect_static:
            with console.status("Collecting static assets", spinner="dots"):
                execute_from_command_line(["manage", "collectstatic", "--no-input"])
    else:
        execute_from_command_line(cmd)
        execute_from_command_line(["manage", "setup_periodic_tasks"])
        if collect_static:
            execute_from_command_line(["manage", "collectstatic", "--no-input"])


    KitchenAIManagement.objects.all().delete()
    try:
        mgmt = KitchenAIManagement.objects.create(
            version = settings.VERSION,
            project_name = "default"
        )
    except Exception as e:
        logger.error(e)
        return


@app.command()
def qcluster() -> None:
    """Run Django-q cluster."""
    from django.core.management import execute_from_command_line
    # execute_from_command_line(["manage", "qcluster", *argv[2:]])
    execute_from_command_line(["manage", "qcluster"])


@app.command()
def runserver(
    module: Annotated[str, typer.Option(help="Python module to load.")] = "",
    ) -> None:
    """Run Django runserver."""
    #NOTE: doing this to reset the sys.argv for gunicorn command.
    sys.argv = [sys.argv[0]]

    django.setup()
    from kitchenai.api import api
    from kitchenai.core.utils import setup

    setup(
        api,
        module=module
    )
    _run_dev_uvicorn(sys.argv)

@app.command()
def run(module: Annotated[str, typer.Option(help="Python module to load.")] = os.environ.get("KITCHENAI_MODULE", "")) -> None:
    """Run Django runserver."""
    sys.argv = [sys.argv[0]]
    django.setup()
    from kitchenai.api import api
    from kitchenai.core.utils import setup

    setup(
        api,
        module=module,
    )

    _run_uvicorn(sys.argv)


@app.command()
def dev(
    address: str ="0.0.0.0:8000", 
    module: Annotated[str, typer.Option(help="Python module to load.")] = "", 
    tailwind: Annotated[bool, typer.Option(help="Tailwind servers.")] = False,
    jupyter: Annotated[bool, typer.Option(help="Jupyter Notebook servers.")] = False,

    ):
    """
    Reads the kitchen config file, reads the application file and runs the KitchenAI server
    """
    import posthog
    import django
    import uuid

    django.setup()
    commands = {"server": "kitchenai runserver"}

    posthog.capture("init", "kitchenai_dev")

    if module:
        commands["server"] = f"kitchenai runserver --module {module} "

    if jupyter:
        #user is running jupyter alongside kitchenai
        from kitchenai.core.models import KitchenAIManagement
        mgmt = KitchenAIManagement.objects.filter(name="kitchenai_management").first()
        notebook_id = uuid.uuid4()
        mgmt.jupyter_token = notebook_id
        mgmt.save()
        
        commands["jupyter"] = f"jupyter lab --NotebookApp.token='{notebook_id}'"



    if tailwind:
        if "django_tailwind_cli" in settings.INSTALLED_APPS:
            commands["tailwind"] = "django-admin tailwind watch"
        if "tailwind" in settings.INSTALLED_APPS:
            commands["tailwind"] = "django-admin tailwind start"
    if "django_q" in settings.INSTALLED_APPS:
        commands["qcluster"] = "kitchenai qcluster"

    typer.echo(f"[INFO] starting development server on {address}")

    # call_command("migrate")
    _run_with_honcho(commands)

@app.command()
def manage(args: list[str] = typer.Argument(None, help="Arguments for Django's manage.py")) -> None:
    """
    Run Django's manage command with additional arguments.
    """
    from django.core.management import execute_from_command_line

    # Build the argument list for Django
    if args is None:
        sys.argv = ["manage"]
    else:
        sys.argv = ["manage"] + args

    execute_from_command_line(sys.argv)

@app.command()
def setup():
    """Run some project setup tasks"""
    django.setup()
    from django.core.management import execute_from_command_line
    import os

    execute_from_command_line(["manage", "migrate"])
    execute_from_command_line(["manage", "setup_periodic_tasks"])

    # Set environment variables for superuser credentials
    email = os.environ.get("DJANGO_SUPERUSER_EMAIL", "admin@localhost")
    password = os.environ.get("DJANGO_SUPERUSER_PASSWORD", "admin")
    username = os.environ.get("DJANGO_SUPERUSER_USERNAME", email.split("@")[0])

    if password == "admin":
        #set it
        os.environ["DJANGO_SUPERUSER_PASSWORD"] = "admin"
    execute_from_command_line(
        ["manage", "createsuperuser", "--noinput", "--traceback", "--email", email, "--username", username]
    )

@app.command()
def build(
    dir: str,
    module: str,
    admin: Annotated[bool, typer.Option("--admin/--no-admin", help="Admin status (default is True)")] = False,
):
    """
    Reads the kitchen config file, reads the application file and runs the KitchenAI server.
    """
    django.setup()
    from django.template import loader
    import pathlib
    import subprocess
    from rich.text import Text

    base_dir = pathlib.Path(dir)

    # Flip the admin flag because we want it to default to True unless the flag is passed
    admin = not admin

    module_name = module.split(":")[0]

    # Save the configuration to the database
    template_name = 'build_templates/Dockerfile.tmpl'

    # Check if requirements.txt and module file exist in the directory
    requirements_file = base_dir / 'requirements.txt'
    module_path = base_dir / f"{module_name}.py"

    if not requirements_file.exists() or not module_path.exists():
        console.print("[bold red]Error:[/bold red] Both requirements.txt and the module file must exist in the specified directory.")
        raise typer.Exit(code=1)

    # Context data to pass into the template
    context = {
        'module': module,
        "admin": admin
    }

    try:
        # Load and render the template with the context data
        template = loader.get_template(template_name)
        rendered_content = template.render(context)

        # Write the rendered Dockerfile to the specified directory
        dockerfile_path = base_dir / 'Dockerfile'
        with open(dockerfile_path, 'w') as dockerfile:
            dockerfile.write(rendered_content)

        console.print(Text(f"Dockerfile successfully created at {dockerfile_path}", style="green"))

    except Exception as e:
        console.print(f"[bold red]Error rendering template:[/bold red] {e}", style="bold red")
        raise typer.Exit(code=1)

    # Build the Docker image using the Dockerfile
    try:
        console.print("[cyan]Building Docker image...[/cyan]")
        # Run the Docker build command
        process = subprocess.Popen(
            ["docker", "build", "-t", "kitchenai-app", dir],
            stdout=subprocess.PIPE,
            stderr=subprocess.PIPE,
            text=True,
            bufsize=1,
            universal_newlines=True,
        )

        # Stream output line by line
        for stdout_line in iter(process.stdout.readline, ""):
            console.print(stdout_line.strip())

        process.stdout.close()
        return_code = process.wait()

        # Check if the Docker build was successful
        if return_code == 0:
            console.print("[green]Docker image built successfully![/green]")
        else:
            # Capture and print stderr output in case of an error
            for stderr_line in iter(process.stderr.readline, ""):
                console.print(f"[bold red]{stderr_line.strip()}[/bold red]")
            console.print("[bold red]Docker build failed.[/bold red]")
            raise typer.Exit(code=1)

    except FileNotFoundError:
        console.print("[bold red]Docker is not installed or not available in your PATH.[/bold red]")
        raise typer.Exit(code=1)
    except Exception as e:
        console.print(f"[bold red]Error during Docker build:[/bold red] {e}")
        raise typer.Exit(code=1)

    except FileNotFoundError:
        console.print("[bold red]Docker is not installed or not available in your PATH.[/bold red]")
        raise typer.Exit(code=1)
    except Exception as e:
        console.print(f"[bold red]Error during Docker build:[/bold red] {e}")
        raise typer.Exit(code=1)

@app.command()
def new():
    """
    Reads the kitchen config file, reads the application file and runs the KitchenAI server
    """

    cookiecutter("https://github.com/epuerta9/cookiecutter-cookbook.git", output_dir=".")



def _run_with_honcho(commands: dict):
    from honcho.manager import Manager

    manager = Manager()
    for name, cmd in commands.items():
        manager.add_process(name, cmd)
    try:
        manager.loop()
    finally:
        manager.terminate()



def _run_uvicorn(argv: list) -> None:
    """
    Run gunicorn + uvicorn workers server.
    https://docs.gunicorn.org/en/stable/settings.html
    https://adamj.eu/tech/2021/12/29/set-up-a-gunicorn-configuration-file-and-test-it/
    """

    import multiprocessing
    from gunicorn.app import wsgiapp  # for gunicorn

    workers = multiprocessing.cpu_count() * 2 + 1
    gunicorn_args = [
        "kitchenai.asgi:application",  # Replace WSGI with ASGI app
        "--bind",
        "0.0.0.0:8001",
        # "unix:/run/kitchenai_demo.gunicorn.sock",  # Use this if you're using a socket file
        "--max-requests",
        "1000",
        "--max-requests-jitter",
        "50",
        "--workers",
        str(workers),
        "--worker-class",
        "uvicorn.workers.UvicornWorker",  # Use Uvicorn worker for ASGI
        "--access-logfile",
        "-",
        "--error-logfile",
        "-",
    ]
    argv.extend(gunicorn_args)

    wsgiapp.run()


def _run_dev_uvicorn(argv: list) -> None:
    """
    Run gunicorn + uvicorn workers server.
    https://docs.gunicorn.org/en/stable/settings.html
    https://adamj.eu/tech/2021/12/29/set-up-a-gunicorn-configuration-file-and-test-it/
    """
    from gunicorn.app import wsgiapp  # for gunicorn

    workers = 2
    gunicorn_args = [
        "kitchenai.asgi:application",  # Replace WSGI with ASGI app
        "--bind",
        "0.0.0.0:8001",
        # "unix:/run/kitchenai_demo.gunicorn.sock",  # Use this if you're using a socket file
        "--max-requests",
        "1000",
        "--max-requests-jitter",
        "50",
        "--workers",
        str(workers),
        "--worker-class",
        "uvicorn.workers.UvicornWorker",  # Use Uvicorn worker for ASGI
        "--access-logfile",
        "-",
        "--error-logfile",
        "-",
    ]
    argv.extend(gunicorn_args)

    wsgiapp.run()



import asyncio
import functools
import importlib
import logging
from collections.abc import Callable
from django.http import StreamingHttpResponse
from ninja import Router


from kitchenai.broker import broker

# Set up logging
logging.basicConfig(level=logging.DEBUG)
logger = logging.getLogger(__name__)

class KitchenAIApp:
    def __init__(self, router: Router = None, namespace: str = 'default', default_db: str = "chromadb"):
        """
        A class that allows you to register routes and storage tasks for a given namespace
        """
        self._namespace = namespace
        self._router = router if router else Router()
        self._storage_tasks = {}
        self._storage_delete_tasks = {}
        self._storage_create_hooks = {}
        self._storage_delete_hooks = {}
        self._default_hook = "kitchenai.contrib.kitchenai_sdk.hooks.default_hook"
        self._default_db =  default_db
        self._query_handlers = {}
        self._agent_handlers = {}
        self._embed_tasks= {}
        self._embed_delete_tasks = {}

    # Decorators for different route types
    def query(self, label: str, streaming=False, llama_stack_emit="", **route_kwargs):
        """Query is a decorator for query handlers with the ability to add middleware"""
        def decorator(func, **route_kwargs):
            @functools.wraps(func)
            async def wrapper(*args, **kwargs):
                if streaming:
                    #NOTE: Streaming HTTP response is only a synchronous operation. Temporary solution
                    # async def event_generator():
                    #     async for event in func(*args, **kwargs):
                    #         # Flush each chunk immediately
                    #         yield event
                    def event_generator():
                        # Call the synchronous function and get the generator
                        gen = func(*args, **kwargs)
                        
                        for event in gen:
                            # Yield each chunk formatted as Server-Sent Events
                            yield event


                    result = StreamingHttpResponse(
                        event_generator(),
                        content_type="text/event-stream",
                        headers={
                            'Cache-Control': 'no-cache',
                            'Transfer-Encoding': 'chunked',
                            'X-Accel-Buffering': 'no',
                        }
                    )
                # Non-streaming behavior
                elif asyncio.iscoroutinefunction(func):
                    result =  await func(*args, **kwargs)
                else:
                    loop = asyncio.get_event_loop()
                    result = await loop.run_in_executor(None, lambda: func(*args, **kwargs))
                return result
            self._query_handlers[f"{self._namespace}.{label}"] = wrapper
            return wrapper

        return decorator
    

    def embed(self, label: str, **route_kwargs):
        """Embed is a decorator for embed handlers"""
        def decorator(func):
            # Store the function immediately when the decorator is applied
            func_path = f"{func.__module__}.{func.__name__}"
            self._embed_tasks[f"{self._namespace}.{label}"] = func_path
            @functools.wraps(func)
            def wrapper(*args, **kwargs):
                return func(*args, **kwargs)  # Just execute the function normally
            return wrapper
        return decorator

    def storage(self, label: str, storage_create_hook: str = None):
        """Storage stores the functions in a hashmap and will run them as async tasks based on ingest_label"""
        def decorator(func):
            # Store the function immediately when the decorator is applied
            func_path = f"{func.__module__}.{func.__name__}"
            self._storage_tasks[f"{self._namespace}.{label}"] = func_path
            if storage_create_hook:
                self._storage_create_hooks[f"{self._namespace}.{label}"] = storage_create_hook
            elif self._storage_create_hooks.get(f"{self._namespace}.{label}") != self._default_hook and self._storage_create_hooks.get(f"{self._namespace}.{label}", None):
                pass
            else:
                logger.debug(f"Setting default success hook for {label}")
                self._storage_create_hooks[f"{self._namespace}.{label}"] = self._default_hook

            @functools.wraps(func)
            def wrapper(*args, **kwargs):
                return func(*args, **kwargs)  # Just execute the function normally
            return wrapper
        return decorator

    def storage_delete(self, label: str):
        """Storage stores the functions in a hashmap and will run them as async tasks based on ingest_label"""
        def decorator(func):
            func_path = f"{func.__module__}.{func.__name__}"
            self._storage_delete_tasks[f"{self._namespace}.{label}"] = func_path
            @functools.wraps(func)
            def wrapper(*args, **kwargs):
                return func(*args, **kwargs)  # Just execute the function normally
            return wrapper
        return decorator
    
    def embed_delete(self, label: str):
        """Embed delete stores the functions in a hashmap and will run them as async tasks based on ingest_label"""
        def decorator(func):
            func_path = f"{func.__module__}.{func.__name__}"
            self._embed_delete_tasks[f"{self._namespace}.{label}"] = func_path
            @functools.wraps(func)
            def wrapper(*args, **kwargs):
                return func(*args, **kwargs)  # Just execute the function normally
            return wrapper
        return decorator

    def agent(self, label: str, streaming=False, **route_kwargs):
        """Agent is a decorator for agent handlers with the ability to add middleware"""
        def decorator(func, **route_kwargs):
            @functools.wraps(func)
            async def wrapper(*args, **kwargs):
                if streaming:
                    #NOTE: Streaming HTTP response is only a synchronous operation
                    async def event_generator():
                        async for event in func(*args, **kwargs):
                            # Flush each chunk immediately
                            yield event

                    return StreamingHttpResponse(
                        event_generator(),
                        content_type="text/event-stream",
                        headers={
                            'Cache-Control': 'no-cache',
                            'Transfer-Encoding': 'chunked',
                            'X-Accel-Buffering': 'no',
                        }
                    )
                # Non-streaming behavior
                elif asyncio.iscoroutinefunction(func):
                    return await func(*args, **kwargs)
                else:
                    loop = asyncio.get_event_loop()
                    return await loop.run_in_executor(None, lambda: func(*args, **kwargs))
            self._agent_handlers[f"{self._namespace}.{label}"] = wrapper

            return wrapper

        return decorator

    def storage_create_hook(self, label: str):
        """Hooks are functions that are run after a storage task is successful"""
        def decorator(func):
            hook = f"{func.__module__}.{func.__name__}"

            self._storage_create_hooks[f"{self._namespace}.{label}"] = hook
            @functools.wraps(func)
            def wrapper(*args, **kwargs):
                return func(*args, **kwargs)  # Just execute the function normally
            return wrapper
        return decorator

    def storage_delete_hook(self, label: str):
        """Hooks are functions that are run after a storage task is successful"""
        def decorator(func):
            hook = f"{func.__module__}.{func.__name__}"

            self._storage_delete_hooks[f"{self._namespace}.{label}"] = hook
            @functools.wraps(func)
            def wrapper(*args, **kwargs):
                return func(*args, **kwargs)  # Just execute the function normally
            return wrapper
        return decorator

    def storage_tasks(self, label: str) -> Callable | None:
        """Returns the function associated with a given label"""
        func_path = self._storage_tasks.get(f"{self._namespace}.{label}")
        if func_path:
            module_path, func_name = func_path.rsplit(".", 1)
            module = importlib.import_module(module_path)
            return getattr(module, func_name)
        return None

    def storage_delete_tasks(self, label: str) -> Callable | None:
        """Returns the function associated with a given label"""
        func_path = self._storage_delete_tasks.get(f"{self._namespace}.{label}")
        if func_path:
            module_path, func_name = func_path.rsplit(".", 1)
            module = importlib.import_module(module_path)
            return getattr(module, func_name)
        return None

    def storage_tasks_list(self) -> dict:
        return self._storage_tasks

    def storage_create_hooks(self, label: str) -> Callable | None:
        """Returns the function associated with a given label"""
        func_path = self._storage_create_hooks.get(f"{self._namespace}.{label}")
        if func_path:
            module_path, func_name = func_path.rsplit(".", 1)
            module = importlib.import_module(module_path)
            return getattr(module, func_name)
        return None

    def storage_delete_hooks(self, label: str) -> Callable | None:
        """Returns the function associated with a given label"""
        func_path = self._storage_delete_hooks.get(f"{self._namespace}.{label}")
        if func_path:
            module_path, func_name = func_path.rsplit(".", 1)
            module = importlib.import_module(module_path)
            return getattr(module, func_name)
        return None

    def to_dict(self):
        return {
            "namespace": self._namespace,
            "query_handlers": list(self._query_handlers.keys()),
            "agent_handlers": list(self._agent_handlers.keys()),
            "embed_tasks": list(self._embed_tasks.keys()),
            "embed_delete_tasks": list(self._embed_delete_tasks.keys()),
            "storage_tasks": list(self._storage_tasks.keys()),
            "storage_delete_tasks": list(self._storage_delete_tasks.keys()),
            "storage_create_hooks": list(self._storage_create_hooks.keys()),
            "storage_delete_hooks": list(self._storage_delete_hooks.keys()),
        }       


import logging
import os
import tempfile
from collections.abc import Callable

from django.core.files.storage import default_storage
from kitchenai.core.models import FileObject, EmbedObject
from kitchenai.core.utils import get_core_kitchenai_app
import importlib

logger = logging.getLogger(__name__)

def process_file_task_app(storage_function: Callable, instance: FileObject, *args, **kwargs):
    """
    process file async function for django apps
    """
    return _process_file_task(storage_function, instance, *args, **kwargs)


def process_file_task_core(instance: FileObject, *args, **kwargs):
    """process file async function for core app using storage task"""
    try:
        kitchenai_app = get_core_kitchenai_app()
        f = kitchenai_app.storage_tasks(instance.ingest_label)
        if f:
            return _process_file_task(f, instance)
        else:
            logger.warning(f"No storage task found for {instance.ingest_label}")
    except Exception as e:
        logger.error(f"Error in run_task: {e}")

def _process_file_task(storage_function: Callable, instance: FileObject, *args, **kwargs):
    """process file task"""
    instance.status = FileObject.Status.PROCESSING
    instance.save()
    file = instance.file
    temp_dir = tempfile.mkdtemp()
    _, extension = os.path.splitext(file.name)

    try:
        with default_storage.open(file.name) as f:
            with tempfile.NamedTemporaryFile(dir=temp_dir, suffix=f"_tmp{extension}") as temp_file:

                temp_file.write(f.read())
                # Calculate the size in MB
                file_size_bytes = temp_file.tell()
                file_size_mb = file_size_bytes / (1024 * 1024)  # Convert bytes to MB

                # Log the size for debugging
                logger.info(f"Size of the temporary file: {file_size_mb} MB")

                # Check if file size exceeds 20 MB
                if file_size_mb > 150:
                    logger.error(f"File size {file_size_mb} MB exceeds the 150 MB limit.")
                    raise Exception("File size exceeds 150 MB limit")

                if file_size_mb > 30:
                    logger.warning(f"File size {file_size_mb} MB exceeds the 30 MB limit.")
                    #TODO: add hook to notify other parts of the system

                temp_file.seek(0)
                metadata = {"file_id": instance.pk, "file_name": file.name, "source": "kitchenai_cookbook", "file_label": instance.name}
                result = storage_function(temp_dir, *args, extension=extension, metadata=metadata, **kwargs)


        return {
            "storage_result": result,
            "ingest_label": instance.ingest_label
        }
    except Exception as e:
        instance.status = FileObject.Status.FAILED
        instance.save()
        raise e
    finally:
        instance.status = FileObject.Status.COMPLETED
        instance.save()

#DELETES
def delete_file_task_app(delete_function: Callable, instance: FileObject, *args, **kwargs):
    """
    delete file async function for django apps
    """
    return delete_function(instance, *args, **kwargs)


def delete_file_task_core(instance: FileObject, *args, **kwargs):
    """delete file async function for core app using storage task"""
    try:
        kitchenai_app = get_core_kitchenai_app()
        f = kitchenai_app.storage_delete_tasks(instance.ingest_label)
        if f:
            return f(instance, *args, **kwargs)
        else:
            logger.warning(f"No delete task found for {instance.ingest_label}")
    except Exception as e:
        logger.error(f"Error in run_task: {e}")

def _embed_task(embed_function: Callable, instance: EmbedObject, *args, **kwargs):
    """embed task"""
    instance.status = EmbedObject.Status.PROCESSING
    instance.save()
    metadata = {"object_id": instance.pk, "text": instance.text, "source": "kitchenai_cookbook", "object_label": instance.ingest_label}

    metadata.update(instance.metadata)
    
    try:
        result = embed_function(instance.text, metadata=metadata, **kwargs)

        return {
            "embed_result": result,
            "ingest_label": instance.ingest_label
        }
    except Exception as e:
        instance.status = EmbedObject.Status.FAILED
        instance.save()
        raise e
    finally:
        instance.status = EmbedObject.Status.COMPLETED
        instance.save()


def embed_task_core(instance: EmbedObject, *args, **kwargs):
    """process file async function for core app using storage task"""
    try:
        kitchenai_app = get_core_kitchenai_app()
        f = kitchenai_app._embed_tasks.get(f"{kitchenai_app._namespace}.{instance.ingest_label}")
        if f:
            module_path, func_name = f.rsplit(".", 1)
            module = importlib.import_module(module_path)
            func = getattr(module, func_name)
            return _embed_task(func, instance, **kwargs)
        else:
            logger.warning(f"No embed task found for {instance.ingest_label}")
    except Exception as e:
        logger.error(f"Error in run_task: {e}")

def delete_embed_task_core(instance: EmbedObject, *args, **kwargs):
    """delete embed task for core app"""
    try:
        kitchenai_app = get_core_kitchenai_app()
        f = kitchenai_app._embed_delete_tasks.get(f"{kitchenai_app._namespace}.{instance.ingest_label}")
        if f:
            module_path, func_name = f.rsplit(".", 1)
            module = importlib.import_module(module_path)
            func = getattr(module, func_name)
            return func(instance, *args, **kwargs)
        else:
            logger.warning(f"No delete embed task found for {instance.ingest_label}")
    except Exception as e:
        logger.error(f"Error in run_task: {e}")




import logging
from typing import TypedDict

from llama_index.core import Document
from llama_index.core import SimpleDirectoryReader
from llama_index.readers.file import DocxReader
from llama_index.readers.file import FlatReader
from llama_index.readers.file import HTMLTagReader
from llama_index.readers.file import ImageReader
from llama_index.readers.file import MarkdownReader
from llama_index.readers.file import PandasCSVReader
from llama_index.readers.file import PDFReader
from llama_parse import LlamaParse


logger = logging.getLogger(__name__)
import nest_asyncio; nest_asyncio.apply()



class ParserResponse(TypedDict):
    documents: list[Document]
    is_llama_api: bool

class Parser:

    def __init__(self, api_key: str = None):
        #PDF Reader with `SimpleDirectoryReader`
        self.llama_parse = LlamaParse(
            result_type="markdown",
            api_key=api_key
        ) if api_key else None

        pdf_parser = PDFReader()

        # Docx Reader example
        docx_parser = DocxReader()

        # Flat Reader example
        text_reader = FlatReader()


        # HTML Tag Reader example
        html_reader = HTMLTagReader()


        # Image Reader example
        image_reader = ImageReader()

        # Markdown Reader example
        md_parser = MarkdownReader()


        csv_parser = PandasCSVReader()

        # PyMuPDF Reader example

        self.file_extractor = {
            ".pdf": pdf_parser,
            ".csv": csv_parser,
            ".md": md_parser,
            ".jpg": image_reader,
            ".jpeg": image_reader,
            ".png": image_reader,
            ".html": html_reader,
            ".txt": text_reader,
            ".docx": docx_parser
        }


    def load(self, dir: str, metadata: dict[str,str] = None,  extension=None, file_size_mb=None) -> ParserResponse:
        """
            load uses a filename and simple directory loader to load the PDF
        """

        def add_meta(filename: str) -> dict[str,str]:
            return metadata
            #to keep track if we used the llama api or loaded the file locally. For billing purposes later
        is_llama_api = False
        if file_size_mb:
            if file_size_mb > 150 and self.llama_parse:

                is_llama_api = True
                if extension:
                    #reset the file extractor and try again with llama_parse
                    self.file_extractor[extension] = self.llama_parse

                    reader = SimpleDirectoryReader(
                        dir, file_extractor=self.file_extractor, file_metadata=add_meta
                    )

                    documents = reader.load_data()
            else:
                reader = SimpleDirectoryReader(
                    dir, file_extractor=self.file_extractor, file_metadata=add_meta
                )

                documents = reader.load_data()
        else:

            reader = SimpleDirectoryReader(
                dir, file_extractor=self.file_extractor, file_metadata=add_meta
            )

            documents = reader.load_data()


        if documents:
            if not documents[0].text:
                #use llama parse as a backup
                is_llama_api = True
                logger.warning("documents text was empty, re-trying with llamaparse")
                if extension and self.llama_parse:
                    #reset the file extractor and try again with llama_parse
                    self.file_extractor[extension] = self.llama_parse

                    reader = SimpleDirectoryReader(
                        dir, file_extractor=self.file_extractor, file_metadata=add_meta
                    )

                    documents = reader.load_data()
                else:
                    logger.error("error llama parse and default loaders failed to extract any text from document")
                    raise Exception("error llama parse and default loaders failed to extract any text from document")


        if documents and not documents[0].text:
            logger.error("error llama parse and default loaders failed to extract any text from document")

            #update the project with a total internal fail
            raise Exception("error llama parse and default loaders failed to extract any text from document")

        return {
            "documents": documents,
            "is_llama_api" : is_llama_api
        }


    def load_string(self, body: str, metadata: dict[str,str] = None) -> list[Document]:
        """
            loads a single document
        """
        doc = Document(text=body, metadata=metadata)


        return doc



import logging

from django.core.management.base import BaseCommand
from kitchenai.core.models import KitchenAIManagement
from kitchenai.core.models import KitchenAIModules

logger = logging.getLogger("kitchenai.core.commands")

class Command(BaseCommand):
    help = 'Runs the development server with dynamic route registration'

    def add_arguments(self, parser):
        # Add a positional argument
        parser.add_argument('module', type=str, help='Load kitchenAI app module')


    def handle(self, *args, **options):
        module = options.get("module")


        try:
            mgmt = KitchenAIManagement.objects.get(name="kitchenai_management")
        except KitchenAIManagement.DoesNotExist:
            logging.exception("error kitchenai database has not been initialized.")
            logging.exception("try kitchenai init")
            return

        #Add the module to the module table
        try:
            module = KitchenAIModules.objects.create(name=module, kitchen=mgmt)
        except Exception:
            logging.exception("error module already loaded.")
            return

        logger.info(f"module loaded: {module}")



from ninja import File
from ninja import Router
from ninja import Schema
from ninja.errors import HttpError
from ninja.files import UploadedFile
from ninja import Schema
from .models import FileObject, EmbedObject
from .utils import get_core_kitchenai_app
from django.http import HttpResponse
import posthog
import logging
from django.apps import apps
from typing import List

logger = logging.getLogger(__name__)
router = Router()

# Create a Schema that represents FileObject
class FileObjectSchema(Schema):
    name: str
    ingest_label: str | None = None
    metadata: dict[str, str] | None = None
    # Add any other fields from your FileObject model that you want to include
class FileObjectResponse(Schema):
    id: int
    name: str
    ingest_label: str
    metadata: dict[str,str]
    status: str

@router.get("/health")
async def default(request):
    return {"msg": "ok"}


@router.post("/file", response=FileObjectResponse)
async def file_upload(request, data: FileObjectSchema,file: UploadedFile = File(...)):
    """main entry for any file upload. Will upload via django storage and emit signals to any listeners"""
    file_object = await FileObject.objects.acreate(
        name=data.name,
        file=file,
        ingest_label=data.ingest_label,
        metadata=data.metadata if data.metadata else {},
        status=FileObject.Status.PENDING
    )
    return file_object


@router.get("/file/{pk}", response=FileObjectResponse)
async def file_get(request, pk: int):
    """get a file"""
    try:
        file_object = await FileObject.objects.aget(pk=pk)
        return file_object
    except FileObject.DoesNotExist:
        raise HttpError(404, "File not found")



@router.delete("/file/{pk}")
async def file_delete(request, pk: int):
    """delete a file"""
    try:    
        await FileObject.objects.filter(pk=pk).adelete()
        return {"msg": "deleted"}
    except FileObject.DoesNotExist:
        raise HttpError(404, "File not found")

@router.get("/file", response=list[FileObjectResponse])
def files_get(request):
    """get all files"""
    file_objects = FileObject.objects.all()
    return file_objects



class EmbedSchema(Schema):
    text: str
    ingest_label: str | None = None
    metadata: dict[str, str] | None = None

    # Add any other fields from your FileObject model that you want to include
class EmbedObjectResponse(Schema):
    id: int
    text: str
    ingest_label: str
    metadata: dict[str,str]
    status: str

#Embed Object API
@router.post("/embed", response=EmbedObjectResponse)
async def embed_create(request, data: EmbedSchema):
    """Create a new embed from text"""
    embed_object = await EmbedObject.objects.acreate(
        text=data.text,
        ingest_label=data.ingest_label,
        metadata=data.metadata if data.metadata else {},
        status=EmbedObject.Status.PENDING,
    )
    return embed_object

@router.get("/embed/{pk}", response=EmbedObjectResponse)
async def embed_get(request, pk: int):
    """Get an embed"""
    try:
        embed_object = await EmbedObject.objects.aget(
            pk=pk,
        )
        return embed_object
    except EmbedObject.DoesNotExist:
        raise HttpError(404, "Embed not found")
    
@router.get("/embed", response=list[EmbedObjectResponse])
def embeds_get(request):
    """Get all embeds"""
    embed_objects = EmbedObject.objects.all()
    return embed_objects    

@router.delete("/embed/{pk}")
async def embed_delete(request, pk: int):
    """Delete an embed"""
    try:
        await EmbedObject.objects.filter(pk=pk).adelete()
        return {"msg": "deleted"}
    except EmbedObject.DoesNotExist:
        raise HttpError(404, "Embed not found")



class QuerySchema(Schema):
    query: str
    metadata: dict[str, str] | None = None

class QueryResponseSchema(Schema):
    response: str

class AgentResponseSchema(Schema):
    response: str

class EmbedSchema(Schema):
    text: str
    metadata: dict[str, str] | None = None


@router.post("/agent/{label}", response=AgentResponseSchema)
async def agent(request, label: str, data: QuerySchema):
    """Create a new agent"""
    try:
        posthog.capture("kitchenai_sdk", "agent_handler")
        core_app = apps.get_app_config("core")
        if not core_app.kitchenai_app:
            logger.error("No kitchenai app in core app config")
            return HttpResponse(status=404)
        agent_func = core_app.kitchenai_app._agent_handlers.get(f"{core_app.kitchenai_app._namespace}.{label}")
        if not agent_func:
            logger.error(f"Agent function not found for {label}")
            return HttpResponse(status=404)

        return await agent_func(data)
    except Exception as e:      
        logger.error(f"Error in agent: {e}")
        return HttpError(500, "agent function not found")

@router.post("/query/{label}", response=QueryResponseSchema)
async def query(request, label: str, data: QuerySchema):
    """Create a new query"""
    """process file async function for core app using storage task"""
    try:
        core_app = apps.get_app_config("core")
        if not core_app.kitchenai_app:
            logger.error("No kitchenai app in core app config")
            return HttpResponse(status=404)
        
        query_func = core_app.kitchenai_app._query_handlers.get(f"{core_app.kitchenai_app._namespace}.{label}")
        if not query_func:
            logger.error(f"Query function not found for {label}")
            return HttpResponse(status=404)
        
        return await query_func(data)
    except Exception as e:
        logger.error(f"Error in query: {e}")
        return HttpError(500, "query function not found")

class KitchenAIAppSchema(Schema):
    namespace: str
    query_handlers: List[str]
    agent_handlers: List[str]
    embed_tasks: List[str]
    embed_delete_tasks: List[str]
    storage_tasks: List[str]
    storage_delete_tasks: List[str]
    storage_create_hooks: List[str]
    storage_delete_hooks: List[str]


@router.get("/labels", response=KitchenAIAppSchema)
async def labels(request):
    """Lists all the custom kitchenai labels"""
    core_app = apps.get_app_config("core")
    if not core_app.kitchenai_app:
        logger.error("No kitchenai app in core app config")
        return HttpResponse(status=404)
        
    return core_app.kitchenai_app.to_dict()




from django.contrib import admin

from .models import (
    EmbedObject,
    FileObject,
    KitchenAIManagement,
    KitchenAIModule,
    KitchenAIRootModule,
    CodeFunction,
    CodeImport,
    CodeSetup,
    Notebook
)


@admin.register(KitchenAIManagement)
class KitchenAIAdmin(admin.ModelAdmin):
    pass


@admin.register(FileObject)
class FileObjectAdmin(admin.ModelAdmin):
    pass


@admin.register(EmbedObject)
class EmbedObjectAdmin(admin.ModelAdmin):
    pass


@admin.register(KitchenAIRootModule)
class KitchenAIRootModuleAdmin(admin.ModelAdmin):
    pass

@admin.register(CodeFunction)
class CodeFunctionAdmin(admin.ModelAdmin):
    pass


@admin.register(CodeImport)
class CodeImportAdmin(admin.ModelAdmin):
    pass

@admin.register(CodeSetup)
class CodeSetupAdmin(admin.ModelAdmin):
    pass


@admin.register(Notebook)
class NotebookAdmin(admin.ModelAdmin):
    pass


@admin.register(KitchenAIModule)
class KitchenAIModuleAdmin(admin.ModelAdmin):
    list_display = (
        "created_at",
        "updated_at",
        "name",
        "kitchen",
        "jupyter_path",
        "file",
    )
    list_filter = ("created_at", "updated_at", "kitchen")
    search_fields = ("name",)
    date_hierarchy = "created_at"



from ninja import Schema

class QuerySchema(Schema):
    query: str
    metadata: dict[str, str] | None = None

class QueryResponseSchema(Schema):
    response: str

class AgentResponseSchema(Schema):
    response: str

class EmbedSchema(Schema):
    text: str
    metadata: dict[str, str] | None = None



import sys

import django


def main() -> None:
    from pathlib import Path
    import os

    os.environ.setdefault("DJANGO_SETTINGS_MODULE", "kitchenai.settings")
    current_path = Path(__file__).parent.parent.resolve()
    sys.path.append(str(current_path))

    django.setup()
    from kitchenai.cli.main import app

    app()


# def _run_gunicorn(argv: list) -> None:
#     """
#     Run gunicorn the wsgi server.
#     https://docs.gunicorn.org/en/stable/settings.html
#     https://adamj.eu/tech/2021/12/29/set-up-a-gunicorn-configuration-file-and-test-it/
#     """
#     import multiprocessing
#     from gunicorn.app import wsgiapp

#     workers = multiprocessing.cpu_count() * 2 + 1
#     gunicorn_args = [
#         "kitchenai.wsgi:application",
#         "--bind",
#         "0.0.0.0:8000",
#         # "unix:/run/kitchenai.gunicorn.sock", # uncomment this line and comment the line above to use a socket file
#         "--max-requests",
#         "1000",
#         "--max-requests-jitter",
#         "50",
#         "--workers",
#         str(workers),
#         "--access-logfile",
#         "-",
#         "--error-logfile",
#         "-",
#     ]
#     argv.extend(gunicorn_args)
#     wsgiapp.run()

if __name__ == "__main__":
    main()



# Generated by Django 5.1.2 on 2024-11-29 20:04

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("core", "0003_codefunction_label"),
    ]

    operations = [
        migrations.AlterField(
            model_name="codefunction",
            name="label",
            field=models.CharField(max_length=255),
        ),
    ]



import logging

from kitchenai.core.utils import get_core_kitchenai_app
logger = logging.getLogger(__name__)

def default_hook(task):
    logger.info(f"default_hook: {task.result}")



def process_file_hook_core(task):
    """process file hook for core app. We have to mock the hook function
    because it's not callable from django q."""
    try:
        kitchenai_app = get_core_kitchenai_app()
        hook = kitchenai_app.storage_create_hooks(task.result.get('ingest_label'))
        if hook:
            hook(task)
        else:
            logger.warning(f"No hook found for {task.result.get('ingest_label')}")
    except Exception as e:
        logger.error(f"Error in run_task: {e}")


def delete_file_hook_core(task):
    logger.info(f"delete_file_hook_core: {task.result}")
    try:
        kitchenai_app = get_core_kitchenai_app()
        hook = kitchenai_app.storage_delete_hooks(task.result.get('ingest_label'))
        if hook:
            hook(task)
    except Exception as e:
        logger.error(f"Error in run_task: {e}")



import logging

from django.apps import apps
from kitchenai.core.api import router as core_router
from ninja import NinjaAPI

logger = logging.getLogger(__name__)

api = NinjaAPI(version="0.9.0")


# Ensure `ready()` is called for all apps
apps.get_app_configs()

# Get the app's router from `MyAppConfig` and add it to the main API
api.add_router("/core", core_router)  # Add app's router here



from IPython.core.magic import Magics, magics_class, cell_magic, line_magic
from asgiref.sync import sync_to_async
import os
import django
import hashlib
import asyncio
import nest_asyncio
from llama_index.llms.openai import OpenAI

from contextlib import redirect_stdout, redirect_stderr
import io

# Setup Django and nest_asyncio
os.environ.setdefault("DJANGO_SETTINGS_MODULE", "kitchenai.settings")
django.setup()
nest_asyncio.apply()

from kitchenai.core.models import CodeFunction,Notebook, CodeImport, CodeSetup
from llama_index.core import PromptTemplate
from django.template import loader


@magics_class
class Cook(Magics):

    def __init__(self, shell):
        super().__init__(shell)  # Initialize the base class
        self.project_name = ""  # Define the class attribute here


    async def get_notebook(self):
            from kitchenai.core.models import Notebook

            try:
                existing_entry = await sync_to_async(Notebook.objects.filter(name=self.project_name).first)()
            except Notebook.DoesNotExist:
                raise Notebook.DoesNotExist

            return existing_entry 
    

    @line_magic
    def kitchenai_get_project(self, line):
        from kitchenai.core.models import Notebook

        async def process_code():
            # Check if a CodeFunction with this label exists
            try:
                existing_entry = await sync_to_async(Notebook.objects.filter(name=self.project_name).first)()
            except Notebook.DoesNotExist:
                return "notebook does not exist"

            return existing_entry


        # Run the async process
        loop = asyncio.get_event_loop()
        if loop.is_running():
            nest_asyncio.apply()
            result = loop.run_until_complete(process_code())  

    @line_magic
    def kitchenai_set_project(self, line):
        """
        Set the project name for the Cook magic commands.
        Usage: %set_project_name MyProject
        """
        
        if not line.strip():
            return "Error: Project name cannot be empty."

        self.project_name = line.strip()
        async def process_code():
            # Check if a CodeFunction with this label exists
            existing_entry = await sync_to_async(Notebook.objects.filter(name=self.project_name).first)()

            if existing_entry:
                # If the hash matches, do nothing
                return f"Project {self.project_name} already exists."

            # Create a new entry
            new_entry = Notebook(name=self.project_name)
            await sync_to_async(new_entry.save)()
            return f"Registered new project {self.project_name}"

        # Run the async process
        loop = asyncio.get_event_loop()
        if loop.is_running():
            nest_asyncio.apply()
            result = loop.run_until_complete(process_code())
        else:
            result = asyncio.run(process_code())


        return f"Project name set to '{self.project_name}'."


    @cell_magic
    def kitchenai_register(self, line, cell):
        """
        Custom magic command to register a code block with a type (storage, query, etc.)
        and a user-defined label.
        """

        # Parse the type and label from the line (e.g., %%kitchen_register storage my-label)
        parts = line.strip().split(" ")
        if len(parts) != 2:
            return "Usage: %%kitchen_register <type> <label>"

        func_type, label = parts
        func_type = func_type.lower()
        if func_type not in CodeFunction.FuncType.values:
            return f"Invalid function type '{func_type}'. Must be one of: {', '.join(CodeFunction.FuncType.values)}"

        # Hash the cell content
        cell_hash = hashlib.sha256(cell.encode()).hexdigest()

        async def process_code():
            # Check if a CodeFunction with this label exists
            existing_entry = await sync_to_async(CodeFunction.objects.filter(type=func_type, label=label, notebook__name=self.project_name).first)()

            if existing_entry:
                # If the hash matches, do nothing
                if existing_entry.hash == cell_hash:
                    return f"Function '{label}' of type '{func_type}' is already registered with the same code."

                # Otherwise, update the existing entry
                existing_entry.hash = cell_hash
                existing_entry.raw_code = cell
                await sync_to_async(existing_entry.save)()
                return f"Updated function '{label}' of type '{func_type}' with new code."

            try:
                notebook = await sync_to_async(Notebook.objects.get)(name=self.project_name)
            except Notebook.DoesNotExist:
                return f"No notebook found with name '{self.project_name}'. Please create it first."


            # Create a new entry
            new_entry = CodeFunction(hash=cell_hash, raw_code=cell, label=label, type=func_type, notebook=notebook)
            await sync_to_async(new_entry.save)()
            return f"Registered new function '{label}' of type '{func_type}' with hash {cell_hash}."

        # Run the async process
        loop = asyncio.get_event_loop()
        if loop.is_running():
            nest_asyncio.apply()
            result = loop.run_until_complete(process_code())
        else:
            result = asyncio.run(process_code())

        print(f"kitchenai_result: {result}")

        ipython = get_ipython()
        ipython.run_cell(cell)



    @cell_magic
    def kitchenai_import(self, line, cell):
        """
        Custom magic command to handle code imports by checking the hash.
        If the hash matches an existing entry, do nothing.
        If the hash differs or the entry doesn't exist, save it.
        """

        # Hash the cell content
        cell_hash = hashlib.sha256(cell.encode()).hexdigest()

        label =  line.strip()

        async def process_import():
            # Check if a CodeImport entry with this hash exists
            existing_entry = await sync_to_async(CodeImport.objects.filter(hash=cell_hash, label=label, notebook__name=self.project_name).first)()

            if existing_entry:
                # If the hash matches, do nothing
                # If the hash matches, do nothing
                if existing_entry.hash == cell_hash:
                    return f"import '{label}' is already registered with the same code."

                # Otherwise, update the existing entry
                existing_entry.hash = cell_hash
                existing_entry.code = cell

                await sync_to_async(existing_entry.save)()
                return f"Updated import '{label}'with new code."
            
            try:
                notebook = await sync_to_async(Notebook.objects.get)(name=self.project_name)
            except Notebook.DoesNotExist:
                return f"No notebook found with name '{self.project_name}'. Please create it first."

            # Create a new entry
            new_entry = CodeImport(hash=cell_hash, code=cell, notebook=notebook, label=label)
            await sync_to_async(new_entry.save)()
            return f"Registered new CodeImport with hash {cell_hash}."

        # Run the async process
        loop = asyncio.get_event_loop()
        if loop.is_running():
            nest_asyncio.apply()
            result = loop.run_until_complete(process_import())
        else:
            result = asyncio.run(process_import())

        print(f"kitchenai_result: {result}")

        ipython = get_ipython()
        ipython.run_cell(cell)

    


    @cell_magic
    def kitchenai_setup(self, line, cell):
        """
        Custom magic command to handle code setups by checking the hash.
        If the hash matches an existing entry, do nothing.
        If the hash differs or the entry doesn't exist, save it.
        """

        # Hash the cell content
        cell_hash = hashlib.sha256(cell.encode()).hexdigest()

        label = line.strip()

        async def process_setup():
            # Check if a CodeSetup entry with this hash exists
            existing_entry = await sync_to_async(CodeSetup.objects.filter(hash=cell_hash, label=label, notebook__name=self.project_name).first)()

            if existing_entry:
                # If the hash matches, do nothing
                # If the hash matches, do nothing
                if existing_entry.hash == cell_hash:
                    return f"setup '{label}' is already registered with the same code."

                # Otherwise, update the existing entry
                existing_entry.hash = cell_hash
                existing_entry.code = cell

                await sync_to_async(existing_entry.save)()
                return f"Updated setup '{label}'with new code."
            
            # Ensure the associated notebook exists
            try:
                notebook = await sync_to_async(Notebook.objects.get)(name=self.project_name)
            except Notebook.DoesNotExist:
                return f"No notebook found with name '{self.project_name}'. Please create it first."

            # Create a new CodeSetup entry
            new_entry = CodeSetup(hash=cell_hash, code=cell, notebook=notebook)
            await sync_to_async(new_entry.save)()
            return f"Registered new CodeSetup with hash {cell_hash}."

        # Run the async process
        loop = asyncio.get_event_loop()
        if loop.is_running():
            nest_asyncio.apply()
            result = loop.run_until_complete(process_setup())
        else:
            result = asyncio.run(process_setup())

        # Execute the code in the current namespace
        print(f"kitchenai_result: {result}")

        ipython = get_ipython()
        ipython.run_cell(cell)


    @line_magic
    def kitchenai_create_module(self, line):
        """
        Create a kitchenai app.py file from the registered code.
        """
        verbose = line.strip() == "verbose"

        async def process_setup():
            # Check if a CodeSetup entry with this hash exists
            code_setups = await sync_to_async(list)(
                CodeSetup.objects.filter(notebook__name=self.project_name).select_related('notebook')
            )
            code_imports = await sync_to_async(list)(
                CodeImport.objects.filter(notebook__name=self.project_name).select_related('notebook')
            )
            code_functions = await sync_to_async(list)(
                CodeFunction.objects.filter(notebook__name=self.project_name).select_related('notebook')
            )


            context = {
                "code_setups" : code_setups,
                "code_imports": code_imports,
                "code_functions" : code_functions
            }
            
            llm = OpenAI(model="gpt-4")
            kitchenai_few_shot = loader.get_template('build_templates/app.tmpl')
            prompt = loader.get_template('build_templates/cook.tmpl')

            kitchenai_module = loader.get_template("build_templates/cook_jupyter.tmpl")

            #this gets us to the comments with the saved code sections.
            kitchenai_module_rendered = await sync_to_async(kitchenai_module.render)(context=context)


            few_shot_rendered = kitchenai_few_shot.render()

            prompt_rendered = prompt.render()

            cook_prompt_template = PromptTemplate(
                prompt_rendered,
            )

            prompt_with_context = cook_prompt_template.format(context_str=kitchenai_module_rendered, few_shot_example=few_shot_rendered)
            
            if verbose:
                print(f"kitchenai_prompt_with_context: {prompt_with_context}")
                print("--------------------------------")

            response = llm.complete(prompt_with_context)

            if verbose: 
                print(f"kitchenai_response: {response.text}")
                print("--------------------------------")

            # Save as .py file
            with open("app.py", "w", encoding="utf-8") as f:
                f.write(response.text)


            # Create a new CodeSetup entry
            return f"Created app.py"

        # Run the async process
        loop = asyncio.get_event_loop()
        if loop.is_running():
            nest_asyncio.apply()
            result = loop.run_until_complete(process_setup())
        else:
            result = asyncio.run(process_setup())

        print(f"kitchenai_result: {result}")
